import os, sys,time, tempfile
from types import *
from Numeric import *
from scipy_base.fastumath import *

try:
    import win32pipe
    plot_popen = win32pipe.popen
    default_terminal = 'windows'
except:
    import os
    plot_popen = os.popen
    default_terminal = 'x11'
    #default_terminal = 'png color'

DEBUG = 0

def is1D(a):
    as = shape(a)
    if(len(as) == 1):
        return 1
    if(as[0] == 1 or as[1]==1):
        return 1
    return 0
    
def row(a):
    return reshape(asarray(a),[1,-1])
def col(a):
    return reshape(asarray(a),[-1,1])

SizeMismatch = 'SizeMismatch'
SizeError = 'SizeError'
NotImplemented = 'NotImplemented'

#b = array((0.,0.,0.))
#b1= array((0.,1.,-1))
#bad = b1/b     
#INF = bad[1]
#NEG_INF = bad[2]
b = array((0.,0.))
b1= array((1.,-1))
bad = b1/b      
INF = bad[0]
NEG_INF = bad[1]
BIG = 1e20
SMALL = 1e-20
tmp_x=0


class Plot:
    m_rmax = SMALL  # used to set up polar plots
    m_rmin = BIG    # used to set up polar plots
    
    m_texthandle = 1
    # I have a little C utility program that starts wgnuplot on win32 machines
    # and redirects stdin to the command window of the program.  While the path 
    # of the program can change, I think it will only work if the name of the file
    # is wgnuplot because of how the program goes about finding the command window.
    # Use 'gnuplot' to specify the path of your wgnuplot program.  Also, to help
    # with debugging, you can specify the '-v' arg so that the wgnuplot window is 
    # visible. Otherwise the window is hidden. 
    if sys.platform == 'win32':
        d,f = os.path.split(__file__)
        gnuhelper = os.path.join(d,'gnuplot_helper.exe')
        gnuplot = os.path.join(d,'wgnuplot.exe')
    else:
        gnuhelper = ''
        gnuplot = 'gnuplot'    
###Eric's configurations
###GUI gnuplot
#   gnuhelper = 'c:\programs\gnuplot\gnuplot.exe'
#   gnuplot = 'c:\programs\gnuplot\wgnuplot.exe'
###console gnuplot  
#   gnuhelper = ''
#   gnuplot = 'c:\programs\gnuplot\gnuplot2.exe'
#unix???
#localhost' # doesn't need a hlper function
#   gnuplot = 'gnuplot'
###David's configurations
# I think you have the console version
### GUI gnuplot
#   gnuhelper = 'c:\gnuplot\gnuplot.exe'
#   gnuplot = 'c:\gnuplot\wgnuplot.exe'
### console gnuplot
#   gnuhelper = ''
#   gnuplot = 'c:\gnuplot\gnuplot.exe'

    def __init__(self, *cmd):
        visible = ''
        if (len(cmd) == 1 and type(cmd[0]) == StringType):
            if(cmd[0][0] == 'v' or cmd[0][0] == 'V' or DEBUG == 1):
                visible = ' -v'
        helper = ''
        if self.gnuhelper != '':
            helper = self.gnuhelper + visible + ' ' 
#       self.g = plot_popen(helper + self.gnuplot, 'w')
#unix???    borrowed from Konrad Hinsen's gnuplot interface
#        self.g = plot_popen(helper + self.gnuplot  + ' -persist 1> /dev/null 2>&1', 'w')
        if sys.platform == 'win32':
            self.g = plot_popen(helper + ' ' + self.gnuplot , 'w')
        else: 
            self.g = plot_popen(helper + self.gnuplot  + ' -persist 1> /dev/null 2>&1', 'w')
        self._defaults()
        time.sleep(.2) # put a pause here because in windows, wgnuplot doesn't seem to get the first plot message
        if (len(cmd) > 0 and type(cmd[0]) != StringType):
            self.plot(cmd)
        self.m_tmpfiles = []    
        self.m_label_coord = 'first'
        self.m_angle = 'radians'
        self.m_mapping = 'cartesian'
        self.m_hold = 0
    def valid(self): 
        try:
            # no idea why we need to send so much garbage. 
            self.g.write('  \n') #test the pipe
            self.g.flush()
            self._send('   \n')
            self._send('   \n')
        except (IOError, ValueError): 
            return 0 
        return 1
    def plot_func(self,func):
        self._init_plot()
        if(type(func) == StringType):
            self._send('plot '+ func)
        else:
            print 'error: requires a function as a string'
        
            
#same as plot, but makes a polar plot 
    def plot(self,*data):
        self._init_plot()
        self._plot(data)    

    def polar(self,*data):
        self._init_plot()
        self._plot(data)    
        #m_rmax is calcalculated in _prepare_plot
        #so this has to be done after it is called
        #!requires a replot - might restructure
        # _plot code here so we don't have to plot
        # twice here
        self._init_polar()

    def _init_polar(self):
        cmd = 'set size square;'
        cmd = cmd + 'set label "90" at graph .5, .98 center;'
        cmd = cmd + 'set label "270" at graph .5, .02 center;'
        cmd = cmd + 'set label "0" at graph .98, .5 center;'
        cmd = cmd + 'set label "180" at graph 0.02, 0.5 center;'
        cmd = cmd + 'set polar;'
        if(self.m_angle == 'radians'):
            ttic = pi/180 * 15
        else:
            ttic = 15       
        cmd = cmd + 'set grid polar %f;' % ttic
        cmd = cmd + 'set noborder;'     
        #need more itellignet choice for tic marks
        #shouldn't gnuplot handle this a bit better??
        mn,mx,inc = self._figure_rtics(self.m_rmin,self.m_rmax,5)
        psize = mx * 1.1
        cmd = cmd + 'set xrange [%f:%f];' % (-psize,psize)
        cmd = cmd + 'set yrange [%f:%f];' % (-psize,psize)
        cmd = cmd + 'set rrange [%f:%f];' % (mn,mx)
        cmd = cmd + 'set xtics axis nomirror;'
        cmd = cmd + 'set xtics %f,%f,%f;' % (-mx,inc,mx)
        cmd = cmd + 'set noytics;'
#       cmd = cmd + 'set ytics axis nomirror;'
#       cmd = cmd + 'set ytics %f,%f,%f;' % (mn,inc,mx)
        cmd = cmd + 'set clip;'
#       self._send(cmd)
        self._replot(cmd)

    def _plot(self,data):
        remains = data
        counter = 0
        plotcmd = ''
        tmpf = []
        while len(remains) > 0:
            group, remains = self._get_plot_group(remains)
            if(len(group) > 0):
                try:
                    cmd, file = self._prepare_plot(group)
                except SizeMismatch, data:                  
                    print "Warning: ignoring plot group ", counter
                    print "The arrays should have the same number of " + data[0]
                    print 'array sizes: ', data[1],'and', data[2]
                else:
                    plotcmd = plotcmd + cmd
                    tmpf.append(file)                   
            counter = counter + 1
            
        # if the plot command was valid, and we aren't
        # adding to the current plot, delete all the old plot files 
        if(len(plotcmd) > 0):
            if(not self.m_hold):            
                self._delete_files()
            plotcmd = plotcmd[0:-2] # get rid of trailing comma
            if DEBUG == 1:
                print plotcmd
            if(self.m_hold == 1 and len(self.m_tmpfiles) > 0):
                self._send('replot ' + plotcmd)         
            else:               
                self._send('plot ' + plotcmd)           
            self.m_tmpfiles = self.m_tmpfiles + tmpf
        
    def _prepare_plot(self, group):
        plotinfo = ''
        x = group[0]
        ar_num = 1
        if(len(group) > 1):
            if( type(group[1]) == StringType):
                plotinfo = group[1]
            else:
                ar_num = 2
                y = group[1]                
        if(len(group) == 3):
            plotinfo = group[2]
        
#force 1D arrays to 2D columns
        if(is1D(x) ):
            x = col(x)
        if(ar_num == 2 and is1D(y) ):
            y = col(y)

        xs = shape(x)
        if(ar_num == 2):
            ys = shape(y)   
#test that x and y have compatible shapes
        if(ar_num == 2):
            #check that each array has the same number of rows
            if(xs[0] != ys[0] ):
                raise SizeMismatch, ('rows', xs, ys)
            #check that x.cols = y.cols
            #no error x has 1 column
            if(xs[1] > 1 and xs[1] != ys[1]):
                raise SizeMismatch, ('cols', xs, ys)
#now write the data to a temp file and then plot it         
        filename = tempfile.mktemp()
        f = open(filename, 'w')
        using = ()
        #plot x against index
        if(ar_num == 1):
            mn,mx = self._find_min_max(x)
            if(mn < self.m_rmin):
                self.m_rmin = mn
            if(mx > self.m_rmax):
                self.m_rmax = mx        
            for i in range(0,xs[0]):
                for j in range(0,xs[1] ):
                    f.write( self._format( (x[i][j],) ))
                f.write('\n')           
            for j in range(0,xs[1] ):                       
                using = using + (' using 0:%d ' % (j+1),)
        #plot x vs y                    
        elif(ar_num ==2):
            mn,mx = self._find_min_max(y)
            if(mn < self.m_rmin):
                self.m_rmin = mn
            if(mx > self.m_rmax):
                self.m_rmax = mx
            
            #x is effectively 1D
            if(xs[1] == 1):
                for i in range(0,xs[0] ):
#                   f.write( "%g " % x[i][0])
                    f.write( self._format( (x[i][0],) ))
                    for j in range(0,ys[1] ):
#                       f.write( "%g " % y[i][j])
                        f.write( self._format( (y[i][j],) ))
                    f.write('\n')           
                for j in range(0,ys[1] ):
                    using = using + (' using 1:%d ' % (j+2),)
            #x is 2D                    
            else:
                for i in range(0,xs[0] ):
                    for j in range(0,xs[1] ):
#                       f.write( "%g %g " % (x[i][j], y[i][j]))
                        f.write( self._format( (x[i][j],y[i][j]) ) )
                    f.write('\n')           
                for j in range(0,xs[1] ):
                    using = using + (' using %d:%d ' % (j*2+1,j*2+2),)
        if plotinfo == '':
            #defualt to printing with lines 
            plotinfo = " notitle w l "
        cmd = ''
        # gnuplot doesn't seem to like backslashes
        fn =    string.replace(filename,'\\','/')
        for us in using:
            cmd = cmd + ' "' + fn + '" ' + us + plotinfo + ', '
        return cmd, filename    
        
    def _get_plot_group(self,data):
        group = ()
        remains = data
        state = 0
        finished = 0
        while(len(remains) > 0 and not finished):
            el = remains[0]
            if(state == 0):
                el = asarray(el)
                state = 1 
            elif(state == 1):
                if(type(el) == StringType):
                    finished = 1
                else:
                    el = asarray(el)
                state = 2
            elif(state == 2):
                finished = 1
                if(type(el) != StringType):
                    break                   
            group = group + (el,)
            remains = remains[1:]       
        return group, remains           

    def autoscale(self):
        self._replot('set autoscale xy')

    def _log(self,axis,st='on'):
        from string import upper
        s = upper(st)
        if(s == 'ON'):
           self._replot('set logscale %s' % axis)
        elif(s == 'OFF'):
            self._replot('set nologscale %s' % axis)
        else:
            print 'Command ' + s + 'is invalid.\n use "on" or "off."'           
            return

    def logx(self,st='on'):
        self._log('x',st)

    def logy(self,st='on'):
        self._log('y',st)
         
    def _set_range(self,axis,rng):
        if(rng == 'auto'):
            cmd = 'set autoscale ' + axis       
        else:
            cmd = 'set ' + axis + 'range ' + ' [%g:%g] ' % (rng[0],rng[1])      
        self._replot(cmd)   
    def xaxis(self,rng):
        self._set_range('x',rng)    
    def yaxis(self,rng):
        self._set_range('y',rng)    
    def zaxis(self,rng):
        self._set_range('z',rng)    #not sure that auto will work for z-range
    def raxis(self,rng):
        self._set_range('r',rng)    
    def taxis(self,rng):
        self._set_range('t',rng)    

    def grid(self,st):
        from string import upper
        s = upper(st)
        if(s == 'ON'):
            cmd = 'set grid;'
        elif(s == 'OFF'):
            cmd = 'set nogrid;'
        else:
            print 'Command ' + s + 'is invalid.\n use "on" or "off."'           
            return
        self._replot(cmd+'show grid')
    def hold(self,st):
        from string import upper
        s = upper(st)
        if(s == 'ON'):
            self.m_hold = 1
        elif(s == 'OFF'):
            self.m_hold = 0
        else:
            print 'Command ' + s + 'is invalid.\n use "on" or "off."'           
            return
    def title(self,t):
        cmd = 'set title "' + t + '"'
        self._replot(cmd)

    def xtitle(self,t):
        cmd = 'set xlabel "' + t + '"'
        self._replot(cmd)

    def ytitle(self,t):
        cmd = 'set ylabel "' + t + '"'
        self._replot(cmd)
    def ztitle(self,t):
        cmd = 'set zlabel "' + t + '"'
        self._replot(cmd)
        
#   text( (x,y,z) ,text, *justification, *symbol)   
#later maybe let this handle arrays of text
    def text(self,xyz,*t):
        if(len(xyz) ==2 ):
            point = '%g, %g' % xyz
        elif(len(xyz) ==3 ):    
            point = '%g, %g, %g' % xyz
        else:   
            #should raise exception
            print "invalid size for location vector"
            return
            
        just = ''
        font = ''
        txt=t[0]    
        if(len(t) > 1):
            options = ['center','left','right']
            if self._valid_option(t[1],options):
                just = ' ' + t[1] + ' '
        if(len(t) > 2):
            #no check for correctness of font
            font = ' font "' + t[2] + '"'
        cmd = 'set label ' + ('%d' % self.m_texthandle) + ' "' + txt + '" at ' + self.m_label_coord + ' ' + point + ' ' + just + ' ' + font
        self.m_texthandle = self.m_texthandle + 1
        self._replot(cmd)
        return self.m_texthandle - 1
        
    def label_coord(self,c):
        valid = ['first','second','graph','screen']
        if self._valid_option(c,valid):
            self.m_label_coord = c

    def legend(self,cmd=''):
        if cmd == 'hide':
            self._replot('set nokey')
        else:
            self._replot('set key ' + cmd)

    def close(self):
        self._delete_files()
        if not self.g.closed:
            self._send('exit')
            self.g.close()  

#3d plotting stuff      
#   surf(x,y,z) or surf(z)
    def surf(self,*data):
        self._init_plot()
        cmd, file= self._prepare_surf(data)                 
        self._complete_surf(cmd,file)

#   plot3d(x,y,z)   or plot3d(z)
    def plot3d(self,*data):
        self._init_plot()
        cmd, file= self._prepare_plot3d(data)                   
        self._complete_surf(cmd,file)

        
    def _complete_surf(self,plotcmd,file):
        if(len(plotcmd) > 0):
            if(not self.m_hold):            
                self._delete_files()
                                        
            plotcmd = plotcmd[0:-2] # get rid of trailing comma
            if DEBUG == 1:
                print plotcmd
            if(self.m_hold == 1 and len(self.m_tmpfiles) > 0):
                plotcmd = 'replot ' + plotcmd
            else:       
                plotcmd = 'splot ' + plotcmd
                #force view to default view     
                plotcmd = 'set view 60,30,1,1; ' + plotcmd
            self._send(plotcmd)         
            self.m_tmpfiles.append(file)

            
    def _prepare_surf(self, group):
        plotinfo = ''
        if(len(group) == 1):
            z, = group
            ar_num = 1
        elif(len(group) == 2):
            z,plotinfo = group
            ar_num = 1
        elif(len(group) == 3):
            ar_num = 3
            x,y,z = group
        elif(len(group) == 4):
            ar_num = 3
            x,y,z,plotinfo = group
        else:
            print 'error in _prepare_surf: group can only 4 terms'
            return
        z = asarray(z)
        if(ar_num == 3):
            x = asarray(x)
            y = asarray(y)

#test the shapes of the arrays
        if(ar_num == 1 and is1D(z)):
            z = row(z); #force 1D to 2D 
        zs = shape(z)
        if(len(zs) != 2):
            raise SizeError, ('z is not 2 dimensional', zs)     
        if(ar_num == 3):
            xs = shape(x)
            ys = shape(y)
            if(xs == ys == zs):
                case = 2
            else:   
                case = 1
                if(not is1D(x)):
                    raise SizeError, ('x is not 1 dimensional', xs)     
                if(not is1D(y)):
                    raise SizeError, ('y is not 1 dimensional', ys)
                #force x and y arrays to 1 dimension                
                x = ravel(x)
                y = ravel(y)            
                xs = shape(x)[0]
                ys = shape(y)[0]
    
                #check that each arrays have compatible shapes
                #note: this is sorta backward (put also more visually
                #intuitive) x matches with columns, and y matches with rows
                if(xs != zs[1]):
                    raise SizeMismatch, ('x axis', xs, zs)
                if(ys != zs[0]):
                    raise SizeMismatch, ('y axis', ys, zs)
                                
#now write the data to a temp file and then plot it         
        filename = tempfile.mktemp()
        f = open(filename, 'w')
        # gnuplot doesn't seem to like backslashes
        fn =    string.replace(filename,'\\','/')   
        #plot x against index
        if(ar_num == 1):
            fileformat = ' matrix '             
            for i in range(0,zs[0]):
                for j in range(0,zs[1] ):
                    f.write( "%g " % z[i][j])
                f.write('\n')
        elif(ar_num == 3):
            fileformat = ''             
            if case == 1:
                    for i in range(0,xs):
                        for j in range(0,ys):
                            f.write( self._format( (x[i],y[j],z[j][i]) ) )
                            f.write('\n')
                        f.write('\n')                                       
            else:                   
                for i in range(0,xs[0]):
                    for j in range(0,xs[1]):
#                       f.write( "%g %g %g\n" % (x[i][j], y[i][j], z[i][j]) )               
                        f.write( self._format( (x[i][j], y[i][j], z[i][j]) ))
                        f.write('\n')
                    f.write('\n')
        if plotinfo == '':
            #defualt to printing with lines 
            plotinfo = " notitle w l "                      
        using = ''
        if(self.m_mapping == 'spherical' and ar_num == 3):
            using = 'using ($3*sin($1)*cos($2)):($3*sin($1)*sin($2)):($3*cos($1))'
        elif(self.m_mapping == 'cylindrical' and ar_num == 3):
            using = 'using ($3*cos($1)):($3*sin($1)):2'
        cmd = ' "' +  fn + '" ' + fileformat + ' ' + using + ' ' + plotinfo + ', '
        return cmd, filename    

    def _prepare_plot3d(self, group):
        plotinfo = ''
        data = group[0]
        if type(data) == TupleType or type(data) == ListType:
            data = map(lambda x:asarray(x),data) # convert to list
        elif not (type(data) == ListType):
            data = [asarray(data)]          
        if(len(group) == 2):
            plotinfo = group[1]

#test the shapes of all the arrays
        for datum in data:
            if(shape(datum)[1] != 3):
                raise SizeError, ('the array must have 3 columns', xs)      
#sort the arrays in desending order based on how many entries they have
#this lets us put all the data in a single file
        data.sort(lambda x,y: cmp(-shape(x)[0],-shape(y)[0]) )
        max_entries = shape(data[0])[0];
#open the file
        filename = tempfile.mktemp()
        f = open(filename, 'w')
        # gnuplot doesn't seem to like backslashes
        fn =    string.replace(filename,'\\','/')   
#now write the data to a temp file and then plot it         
        for j in range(0,max_entries):
            for i in range(0,len(data)):
                try:
                    d = data[i][j]
#                   f.write( "%g %g %g " % (d[0],d[1],d[2]) )
                    f.write( self._format( (d[0],d[1],d[2]) ))
                except IndexError:
                    #ignore
                    i=i
            f.write('\n')
        using = ()      
        if DEBUG == 1:
            print self.m_mapping    
        for i in range(0,len(data)):
            us = 'using '
            c = (i*3+1,i*3+2,i*3+3)
            if(self.m_mapping == 'spherical' ):
                us = us + ( '($%d*sin($%d)*cos($%d)):($%d*sin($%d)*sin($%d)):($%d*cos($%d)) ' % 
                         (c[2],   c[0],    c[1],  c[2],   c[0],    c[1],  c[2],   c[0]) ) 
            elif(self.m_mapping == 'cylindrical'):
                using = us + ( '($%d*cos($%d)):($%d*sin($%d)):%d ' %
                           (c[2],  c[0], c[2],  c[0], c[1]) )
            else:               
                us = us + (' %d:%d:%d ' % c)            
            using = using + (us,)
        if plotinfo == '':
            #defualt to printing with lines 
            plotinfo = " notitle w l "  
        cmd = ''                        
        for us in using:
            cmd = cmd + ' "' + fn + '" ' + us + plotinfo + ', '                     
        return cmd, filename    

    def mapping(self,val):
        options = ['cartesian', 'spherical', 'cylindrical']
        if self._valid_option(val,options):
            self.m_mapping = val
#       We'll do the mapping via a ''using' statement
#       because I'd rather use polar-spherical coordinates      
#           self._replot('set mapping ' + val)
            
    def angles(self,val):
        options = ['degrees',  'radians']

        if self._valid_option(val,options):
            self._replot('set angles ' + val)
            self.m_angle = val 
        #set the view of 3d plots val = (xrot, *zrot, *scale, *zscale)
    def view(self,val):
        if(val == 'default'):
            self._replot('set view 60,30,1,1')
            return
            
        if (0 <= val[0] and val[0] <= 180):
            cmd = 'set view ' + ("%f " % val[0] )           
        else:
            print 'error: xrot must be between 0 and 180'           
            return
        if(len(val) > 1):
            zrot = val[1]
            if (0 <= zrot and zrot <= 360):
                cmd = cmd + (",%f " % val[1] )          
            else:
                print 'error: zrot must be between 0 and 360'           
                return
        if(len(val) > 2):
            cmd = cmd + (",%f " % val[2] )          
        if(len(val) > 3):
            cmd = cmd + (",%f " % val[3] )          
        self._replot(cmd)
    # location as percentage of z axis above x,y plane where
    # plotting starts       
    def ticlevel(self,val):
        cmd = 'set ticslevel ' + "%f" % val + ';show tics'
        self._replot(cmd)

    def hidden(self,val):
        options = ['remove',  'show']

        if self._valid_option(val,options):
            no=' o'
            if(val == 'remove'):
                no = '';
            self._replot('set ' + no + 'hidden3d')
    
    def output(self,filename,type,options = ''):
        #changed 7/19/99 ej: always switches to current working directory
        import os, time
        self._send("cd '%s'" % os.path.abspath(os.curdir))
        self._replot('set terminal ' + type + ' ' + options + ";set output '" + filename + "'")
        self._replot('set terminal ' + default_terminal + ';set output')
    def size(self,size=(1.,1.)):
        cmd = 'set size %f,%f' % size
        self._replot(cmd)
    def png(self,filename):
        self.output(filename,'png color')
#contour plotting stuff
#   def contour(self,val):      
                
#utilities
    def _defaults(self):
        #this changes things for the windows terminals
        #just re-arrange some of the colors
        #??? how do I force the defaults to these values
        #??? i.e. can I tell the terminal what I want the 
        #??? default line colors to be?
        cmd ='set linestyle 1 lt 1;'    #bright blue
        cmd = cmd +'set linestyle 2 lt 10;' #dark green
        cmd = cmd +'set linestyle 3 lt 3;'  #red
        cmd = cmd +'set linestyle 4 lt 5;'  #magenta
        cmd = cmd +'set linestyle 5 lt 12;'     #purple
        cmd = cmd +'set linestyle 6 lt 14;'     #cyan
        cmd = cmd +'set linestyle 7 lt 15;'     #yellow
        cmd = cmd +'set linestyle 8 lt 5;'  #dark blue
        cmd = cmd +'set grid;'
        cmd = cmd +'show grid\n'     
        self._send(cmd)
    def _init_plot(self):
        self.m_rmin = BIG
        self.m_rmax = SMALL

        self._send('reset')
        self.grid('on')
        self.angles(self.m_angle)
        
    def _valid_option(self,opt,optlist):
        try:
            optlist.index(opt)
        except ValueError:
            print opt, ' is an invalid choice.'
            print 'Valid choices are ', optlist
            return 0
        else:
            return 1

#used to format data sent to gnuplot files.
#would not be necessary if INF, -INF, and IND
#didn't exist
    def _format(self,x):
        #############################
        ## complications added to check for INF, -INF
        #############################
        #note: does not check for IND at the moment
        str = ''
        for val in x:
            if(val == INF or val == NEG_INF):
                str = str + ' - '
            else:
                str = '%s %g' % (str,val)
        return str
    def _find_min_max(self,x):
        global tmp_x
        #find min...
        temp_min = BIG 
        mn = minimum.reduce(minimum.reduce(x))
        if ( mn < temp_min):
            temp_min = mn

        temp_max = SMALL    
        mx = maximum.reduce(maximum.reduce(x))
        if ( mx > temp_max):
            temp_max = mx
        #############################
        ## complications added to check for INF, -INF
        #############################
        #the following garbage is to check for an infinite or
        #undefined value... there has got to be a smarter way
        # don't compare to 1#IND yet cause it doesn't compare correctly
        if(temp_min == NEG_INF):
            mask = equal(x,NEG_INF)
            tmp_x = choose(mask,(x,BIG))
            mn = minimum.reduce(minimum.reduce(tmp_x))
            if ( mn < temp_min):
                temp_min = mn
        if(temp_max == INF):
            mask = equal(x,INF)
            tmp_x = choose(mask,(x,SMALL))  
            mx = maximum.reduce(maximum.reduce(tmp_x))
            if ( mx < temp_max):
                temp_max = mx
        return (temp_min, temp_max)
    
    #translated from gnuplot code
    def _figure_rtics(self,minval,maxval,guess=20):
        xr = abs(maxval-minval)
        l10 = log10(xr)
        fl = floor(l10)
        xnorm = 10**(l10-fl)
        #you can change the value of 5
        # to something if you want more tics...
        posns = guess / xnorm
        
        if (posns > 40):
            tics = 0.05     # eg 0, .05, .10, ... 
        if (posns > 20):
            tics = 0.1      # eg 0, .1, .2, ... 
        elif (posns > 10):
            tics = 0.2      #/* eg 0,0.2,0.4,... */
        elif (posns > 4):
            tics = 0.5      #/* 0,0.5,1, */
        elif (posns > 1):
            tics = 1        #/* 0,1,2,.... */
        elif (posns > 0.5):
            tics = 2        #/* 0, 2, 4, 6 */
        else:
            # getting desperate... the ceil is to make sure we
            # go over rather than under - eg plot [-10:10] x*x
            # gives a range of about 99.999 - tics=xnorm gives
            # tics at 0, 99.99 and 109.98  - BAD !
            # This way, inaccuracy the other way will round
            # up (eg 0->100.0001 => tics at 0 and 101
            # I think latter is better than former   
            tics = ceil(xnorm);

        tic = tics * 10.0**fl;
        results = [0,0,0]
        results[0] = tic * floor(minval / tic); 
        results[1] = tic * ceil(maxval / tic);  
        results[2] = tic
        return results
        
    def _delete_files(self):
        for file in self.m_tmpfiles:
            try:
                os.unlink(file)
            except:
                #do nothing
                file = file
            else:
                self.m_tmpfiles.remove(file)
    
    def _replot(self,cmd):
        self._send(cmd + ';replot\n')

#this is a compromise: for small commands, send directly to gnuplot
# for large commands, put them in a file and load the file from gnuplot
    def _send(self,cmd):
        if(len(cmd) < 200):
            self.g.write(cmd + '\n')
            self.g.flush()
        else:           
            filename = tempfile.mktemp()
            f = open(filename, 'w')
            f.write(cmd + '\n')
            f.close();
            fn =    string.replace(filename,'\\','/')
            self.g.write('load "' + fn +  '"\n')
            self.g.flush()  
            self.m_tmpfiles.append(filename)
        time.sleep(.15)             
#I'd rather use some function similar to the following to send data to gnuplot
#but I'm having problems with getting all the data transferred to gnuplot in 
#large(huge) plot commands - it hangs after 2042 characters.  I don't know
#if the problem is with the pipe, with gnuplots command line buffer size, or
#my helper program (probable??)     
#NOTE: Using spy++ to watch WM_CHAR messages, I was able to confirm that 
# all the characters are indeed sent to wgnuplot by the gnuhelper program.
# It looks as if wgnuplot message pump just can't handle it if a barrage 
# of events are sent its way.  If I put in huge delays, the problem goes away,
# but for now, I think it is more efficient just to use files.
    def _send_try2_but_still_doesnt_work(self,cmd):
        bufsize = 50
        idx = range(0,len(cmd),bufsize)
        idx.append(len(cmd))
        for i in range(len(idx)-1):         
#           print i, idx[i]+1, idx[i+1]
            print cmd[idx[i]:idx[i+1]]
            self.g.write(cmd[idx[i]:idx[i+1]])
            self.g.flush()  
            time.sleep(.05)         
        self.g.write('\n')
        self.g.flush()      
        #without the sleep command, it appears things get lost in the pipe!
        #I don't understand this
        time.sleep(.15)   
#and really it should be this easy...
    def _send_best_but_doesnt_work(self,cmd):
        self.g.write(cmd + '\n')
        self.g.flush()  
#destructor
    def __del__(self):
        try: self.close()
        except (ValueError, IOError, AttributeError): pass

#this version of prepare surf does all the coordinate mapping in python.
#Initially I thought this was a good idea, but now I have no idea why I thought
#that.  I'll keep it until I'm sure it isn't a good idea
    def _prepare_surf_slower(self, group):
        plotinfo = ''
        if(len(group) == 1):
            z, = group
            ar_num = 1
        elif(len(group) == 2):
            z,plotinfo = group
            ar_num = 1
        elif(len(group) == 3):
            ar_num = 3
            x,y,z = group
        elif(len(group) == 4):
            ar_num = 3
            x,y,z,plotinfo = group
        else:
            print 'error in _prepare_surf: group can only 4 terms'
            return
        z = asarray(z)
        if(ar_num == 3):
            x = asarray(x)
            y = asarray(y)

#test the shapes of the arrays
        if(ar_num == 1 and is1D(z)):
            z = row(z); #force 1D to 2D 
        zs = shape(z)
        if(len(zs) != 2):
            raise SizeError, ('z is not 2 dimensional', zs)     
        if(ar_num == 3):
            xs = shape(x)
            ys = shape(y)
            if(xs == ys == zs):
                case = 2
            else:   
                case = 1
                if(not is1D(x)):
                    raise SizeError, ('x is not 1 dimensional', xs)     
                if(not is1D(y)):
                    raise SizeError, ('y is not 1 dimensional', ys)
                #force x and y arrays to 1 dimension                
                x = ravel(x)
                y = ravel(y)            
                xs = shape(x)[0]
                ys = shape(y)[0]
    
                #check that each arrays have compatible shapes
                #note: this is sorta backward (put also more visually
                #intuitive) x matches with columns, and y matches with rows
                if(xs != zs[1]):
                    raise SizeMismatch, ('x axis', xs, zs)
                if(ys != zs[0]):
                    raise SizeMismatch, ('y axis', ys, zs)
                                
#now write the data to a temp file and then plot it         
        filename = tempfile.mktemp()
        f = open(filename, 'w')
        # gnuplot doesn't seem to like backslashes
        fn =    string.replace(filename,'\\','/')   
        #plot x against index
        if(ar_num == 1):
            fileformat = ' matrix '             
            for i in range(0,zs[0]):
                for j in range(0,zs[1] ):
                    f.write( "%g " % z[i][j])
                f.write('\n')
        elif(ar_num == 3):
            fileformat = ''             
            if case == 1:
                if(self.m_mapping == 'spherical'):
                    sinx = sin(x)
                    cosx = cos(x)       
                    siny = sin(y)
                    cosy = cos(y)
                    for i in range(0,xs):
                        for j in range(0,ys):
                            v1 = z[j][i] * sinx[i]*cosy[j]
                            v2 = z[j][i] * sinx[i]*siny[j]
                            v3 = z[j][i] * cosx[i]
                            f.write( self._format( (v1,v2,v3) ) )
                            f.write('\n')
                        f.write('\n')                       
                elif(self.m_mapping == 'cylindrical'):
                    sinx = sin(x)
                    cosx = cos(x)       
                    for i in range(0,xs):
                        for j in range(0,ys):
                            v1 = z[j][i] * cosx[i]
                            v2 = z[j][i] * sinx[i]
                            v3 = y[j]
                            f.write( self._format( (v1,v2,v3) ) )
                            f.write('\n')
                        f.write('\n')                       
                else:
                    for i in range(0,xs):
                        for j in range(0,ys):
                            v1 = x[i]
                            v2 = y[j]
                            v3 = z[j][i]
                            f.write( self._format( (v1,v2,v3) ) )
                            f.write('\n')
                        f.write('\n')                       
                        
            else:                   
                for i in range(0,xs[0]):
                    for j in range(0,xs[1]):
#                       f.write( "%g %g %g\n" % (x[i][j], y[i][j], z[i][j]) )               
                        f.write( self._format( (x[i][j], y[i][j], z[i][j]) ))
                        f.write('\n')
                    f.write('\n')
        if plotinfo == '':
            #defualt to printing with lines 
            plotinfo = " notitle w l "                      
        using = ''
#       if(self.m_mapping == 'spherical' and ar_num == 3):
#           using = 'using ($3*sin($1)*cos($2)):($3*sin($1)*sin($2)):($3*cos($1))'
#       elif(self.m_mapping == 'cylindrical' and ar_num == 3):
#           using = 'using ($3*cos($1)):($3*sin($1)):2'
        cmd = ' "' +  fn + '" ' + fileformat + ' ' + using + ' ' + plotinfo + ', '
        return cmd, filename    



"""
Data input and output
=====================

  Classes

     fopen -- a class for easily reading and writing binary data.

  Functions

     read_array -- reading ascii streams into Numeric arrays
     write_array -- write an array to an ascii stream
     loadmat -- read a MATLAB style mat file (version 4 and 5)
     savemat -- write a MATLAB (version <= 4) style mat file

     fread -- low-level reading
     fwrite -- low-level writing
     bswap -- in-place byte-swapping
     packbits -- Pack a binary array of 1's and 0's into an array of bytes
     unpackbits -- Unpack an array packed by packbits.

     save --- simple storing of Python dictionary into module
               that can then be imported and the data accessed as
               attributes of the module.

     mminfo   -- query matrix info from Matrix Market formatted file
     mmread   -- read matrix from Matrix Market formatted file
     mmwrite  -- write matrix to Matrix Market formatted file

"""
postpone_import = 1




import sys
from distutils.dist import Distribution as OldDistribution
from distutils.errors import DistutilsSetupError
from types import *

class Distribution (OldDistribution):

    if sys.version[:3]<'2.2':
        # For backward compatibility. Use libraries
        # instead of fortran_libraries.
        fortran_libraries = None

    def return_false(self):
        # Used by build_ext.run()
        return 0

    def check_data_file_list(self):
        """Ensure that the list of data_files (presumably provided as a
           command option 'data_files') is valid, i.e. it is a list of
           2-tuples, where the tuples are (name, list_of_libraries).
           Raise DistutilsSetupError if the structure is invalid anywhere;
           just returns otherwise."""
        print 'check_data_file_list'
        if type(self.data_files) is not ListType:
            raise DistutilsSetupError, \
                  "'data_files' option must be a list of tuples"

        for lib in self.data_files:
            if type(lib) is not TupleType and len(lib) != 2:
                raise DistutilsSetupError, \
                      "each element of 'data_files' must a 2-tuple"

            if type(lib[0]) is not StringType:
                raise DistutilsSetupError, \
                      "first element of each tuple in 'data_files' " + \
                      "must be a string (the package with the data_file)"

            if type(lib[1]) is not ListType:
                raise DistutilsSetupError, \
                      "second element of each tuple in 'data_files' " + \
                      "must be a list of files."
        return
   
    def get_data_files (self):
        print 'get_data_files'
        self.check_data_file_list()
        filenames = []
        
        # Gets data files specified
        for ext in self.data_files:
            filenames.extend(ext[1])

        return filenames



#!/usr/bin/env python
"""
Postpone module import to future.

Python versions: 1.5.2 - 2.3.x
Author: Pearu Peterson <pearu@cens.ioc.ee>
Created: March 2003
$Revision: 1.26 $
$Date: 2004/09/28 19:41:08 $
"""
__all__ = ['ppimport','ppimport_attr','ppresolve']

import os
import sys
import string
import types
import traceback

DEBUG=0

_ppimport_is_enabled = 1
def enable():
    """ Enable postponed importing."""
    global _ppimport_is_enabled
    _ppimport_is_enabled = 1

def disable():
    """ Disable postponed importing."""
    global _ppimport_is_enabled
    _ppimport_is_enabled = 0

class PPImportError(ImportError):
    pass

def _get_so_ext(_cache={}):
    so_ext = _cache.get('so_ext')
    if so_ext is None:
        if sys.platform[:5]=='linux':
            so_ext = '.so'
        else:
            try:
                # if possible, avoid expensive get_config_vars call
                from distutils.sysconfig import get_config_vars
                so_ext = get_config_vars('SO')[0] or ''
            except ImportError:
                #XXX: implement hooks for .sl, .dll to fully support
                #     Python 1.5.x   
                so_ext = '.so'
        _cache['so_ext'] = so_ext
    return so_ext

def _get_frame(level=0):
    try:
        return sys._getframe(level+1)
    except AttributeError:
        # Python<=2.0 support
        frame = sys.exc_info()[2].tb_frame
        for i in range(level+1):
            frame = frame.f_back
        return frame

def ppimport_attr(module, name):
    """ ppimport(module, name) is 'postponed' getattr(module, name)
    """
    global _ppimport_is_enabled
    if _ppimport_is_enabled and isinstance(module, _ModuleLoader):
        return _AttrLoader(module, name)
    return getattr(module, name)

class _AttrLoader:
    def __init__(self, module, name):
        self.__dict__['_ppimport_attr_module'] = module
        self.__dict__['_ppimport_attr_name'] = name

    def _ppimport_attr_getter(self):
        module = self.__dict__['_ppimport_attr_module']
        if isinstance(module, _ModuleLoader):
            # in case pp module was loaded by other means
            module = sys.modules[module.__name__]
        attr = getattr(module,
                       self.__dict__['_ppimport_attr_name'])
        try:
            d = attr.__dict__
            if d is not None:
                self.__dict__ = d
        except AttributeError:
            pass
        self.__dict__['_ppimport_attr'] = attr

        return attr

    def __nonzero__(self):
        return 1

    def __getattr__(self, name):
        try:
            attr = self.__dict__['_ppimport_attr']
        except KeyError:
            attr = self._ppimport_attr_getter()
        if name=='_ppimport_attr':
            return attr
        return getattr(attr, name)

    def __repr__(self):
        if self.__dict__.has_key('_ppimport_attr'):
            return repr(self._ppimport_attr)
        module = self.__dict__['_ppimport_attr_module']
        name = self.__dict__['_ppimport_attr_name']
        return "<attribute %s of %s>" % (`name`,`module`)

    __str__ = __repr__

    # For function and class attributes.
    def __call__(self, *args, **kwds):
        return self._ppimport_attr(*args,**kwds)



def _is_local_module(p_dir,name,suffices):
    base = os.path.join(p_dir,name)
    for suffix in suffices:
        if os.path.isfile(base+suffix):
            if p_dir:
                return base+suffix
            return name+suffix

def ppimport(name):
    """ ppimport(name) -> module or module wrapper

    If name has been imported before, return module. Otherwise
    return ModuleLoader instance that transparently postpones
    module import until the first attempt to access module name
    attributes.
    """
    global _ppimport_is_enabled

    level = 1
    parent_frame = p_frame = _get_frame(level)
    while not p_frame.f_locals.has_key('__name__'):
        level = level + 1
        p_frame = _get_frame(level)

    p_name = p_frame.f_locals['__name__']
    if p_name=='__main__':
        p_dir = ''
        fullname = name
    elif p_frame.f_locals.has_key('__path__'):
        # python package
        p_path = p_frame.f_locals['__path__']
        p_dir = p_path[0]
        fullname = p_name + '.' + name
    else:
        # python module
        p_file = p_frame.f_locals['__file__']
        p_dir = os.path.dirname(p_file)
        fullname = p_name + '.' + name

    # module may be imported already
    module = sys.modules.get(fullname)
    if module is not None:
        if _ppimport_is_enabled or isinstance(module, types.ModuleType):
            return module
        return module._ppimport_importer()

    so_ext = _get_so_ext()
    py_exts = ('.py','.pyc','.pyo')
    so_exts = (so_ext,'module'+so_ext)

    for d,n,fn,e in [\
        # name is local python module or local extension module
        (p_dir, name, fullname, py_exts+so_exts),
        # name is local package
        (os.path.join(p_dir, name), '__init__', fullname, py_exts),
        # name is package in parent directory (scipy specific)
        (os.path.join(os.path.dirname(p_dir), name), '__init__', name, py_exts),
        ]:
        location = _is_local_module(d, n, e)
        if location is not None:
            fullname = fn
            break

    if location is None:
        # name is to be looked in python sys.path.
        fullname = name
        location = 'sys.path'

    # Try once more if module is imported.
    # This covers the case when importing from python module
    module = sys.modules.get(fullname)

    if module is not None:
        if _ppimport_is_enabled or isinstance(module,types.ModuleType):
            return module
        return module._ppimport_importer()
    # It is OK if name does not exists. The ImportError is
    # postponed until trying to use the module.

    loader = _ModuleLoader(fullname,location,p_frame=parent_frame)
    if _ppimport_is_enabled:
        return loader

    return loader._ppimport_importer()

def _get_frame_code(frame):
    filename = frame.f_code.co_filename
    lineno = frame.f_lineno
    result = '%s in %s:\n' % (filename,frame.f_code.co_name)
    if not os.path.isfile(filename):
        return result
    f = open(filename)
    i = 1
    line = f.readline()
    while line:
        line = f.readline()
        i = i + 1
        if (abs(i-lineno)<2):
            result += '#%d: %s\n' % (i,line.rstrip())
        if i>lineno+3:
            break
    f.close()
    return result

def frame_traceback(frame):
    if not frame:
        return
    blocks = []
    f = frame
    while f:
        blocks.insert(0,_get_frame_code(f))
        f = f.f_back
    print '='*50
    print '\n'.join(blocks)
    print '='*50    

class _ModuleLoader:
    # Don't use it directly. Use ppimport instead.

    def __init__(self,name,location,p_frame=None):

        # set attributes, avoid calling __setattr__
        self.__dict__['__name__'] = name
        self.__dict__['__file__'] = location
        self.__dict__['_ppimport_p_frame'] = p_frame

        if location != 'sys.path':
            from scipy_test.testing import ScipyTest
            self.__dict__['test'] = ScipyTest(self).test

        # install loader
        sys.modules[name] = self

    def _ppimport_importer(self):
        name = self.__name__

        try:
            module = sys.modules[name]
	except KeyError:
            raise ImportError,self.__dict__.get('_ppimport_exc_info')[1]
        if module is not self:
            exc_info = self.__dict__.get('_ppimport_exc_info')
            if exc_info is not None:
                raise PPImportError,\
                      ''.join(traceback.format_exception(*exc_info))
            else:
                assert module is self,`(module, self)`

        # uninstall loader
        del sys.modules[name]

        if DEBUG:
            print 'Executing postponed import for %s' %(name)
        try:
            module = __import__(name,None,None,['*'])
        except Exception,msg: # ImportError:
            if DEBUG:
                p_frame = self.__dict__.get('_ppimport_p_frame',None)
                frame_traceback(p_frame)
            self.__dict__['_ppimport_exc_info'] = sys.exc_info()
            raise

        assert isinstance(module,types.ModuleType),`module`

        self.__dict__ = module.__dict__
        self.__dict__['_ppimport_module'] = module

        # XXX: Should we check the existence of module.test? Warn?
        from scipy_test.testing import ScipyTest
        module.test = ScipyTest(module).test

        return module

    def __setattr__(self, name, value):
        try:
            module = self.__dict__['_ppimport_module']
        except KeyError:
            module = self._ppimport_importer()
        return setattr(module, name, value)

    def __getattr__(self, name):
        try:
            module = self.__dict__['_ppimport_module']
        except KeyError:
            module = self._ppimport_importer()
        return getattr(module, name)

    def __repr__(self):
        global _ppimport_is_enabled
        if not _ppimport_is_enabled:
            try:
                module = self.__dict__['_ppimport_module']
            except KeyError:
                module = self._ppimport_importer()
            return module.__repr__()
        if self.__dict__.has_key('_ppimport_module'):
            status = 'imported'
        elif self.__dict__.has_key('_ppimport_exc_info'):
            status = 'import error'
        else:
            status = 'import postponed'
        return '<module %s from %s [%s]>' \
               % (`self.__name__`,`self.__file__`, status)

    __str__ = __repr__

def ppresolve(a,ignore_failure=None):
    """ Return resolved object a.

    a can be module name, postponed module, postponed modules
    attribute, string representing module attribute, or any
    Python object.
    """
    global _ppimport_is_enabled
    if _ppimport_is_enabled:
        disable()
        a = ppresolve(a,ignore_failure=ignore_failure)
        enable()
        return a
    if type(a) is type(''):
        ns = a.split('.')
        if ignore_failure:
            try:
                a = ppimport(ns[0])
            except:
                return a
        else:
            a = ppimport(ns[0])
        b = [ns[0]]
        del ns[0]
        while ns:
            if hasattr(a,'_ppimport_importer') or \
                   hasattr(a,'_ppimport_module'):
                a = a._ppimport_module
            if hasattr(a,'_ppimport_attr'):
                a = a._ppimport_attr
            b.append(ns[0])
            del ns[0]
            if ignore_failure and not hasattr(a, b[-1]):
                a = '.'.join(ns+b)
                b = '.'.join(b)
                if sys.modules.has_key(b) and sys.modules[b] is None:
                    del sys.modules[b]
                return a
            a = getattr(a,b[-1])
    if hasattr(a,'_ppimport_importer') or \
           hasattr(a,'_ppimport_module'):
        a = a._ppimport_module
    if hasattr(a,'_ppimport_attr'):
        a = a._ppimport_attr
    return a

def _ppresolve_ignore_failure(a):
    return ppresolve(a,ignore_failure=1)

try:
    import pydoc as _pydoc
except ImportError:
    _pydoc = None

if _pydoc is not None:
    # Redefine __call__ method of help.__class__ to
    # support ppimport.
    import new as _new

    _old_pydoc_help_call = _pydoc.help.__class__.__call__
    def _scipy_pydoc_help_call(self,*args,**kwds):
        _old_pydoc_help_call.__doc__
        return _old_pydoc_help_call(self, *map(_ppresolve_ignore_failure,args),
                                    **kwds)
    _pydoc.help.__class__.__call__ = _new.instancemethod(_scipy_pydoc_help_call,
                                                         None,
                                                         _pydoc.help.__class__)

    _old_pydoc_Doc_document = _pydoc.Doc.document
    def _scipy_pydoc_Doc_document(self,*args,**kwds):
        _old_pydoc_Doc_document.__doc__
        args = (_ppresolve_ignore_failure(args[0]),) + args[1:]
        return _old_pydoc_Doc_document(self,*args,**kwds)
    _pydoc.Doc.document = _new.instancemethod(_scipy_pydoc_Doc_document,
                                              None,
                                              _pydoc.Doc)

    _old_pydoc_describe = _pydoc.describe
    def _scipy_pydoc_describe(object):
        _old_pydoc_describe.__doc__
        return _old_pydoc_describe(_ppresolve_ignore_failure(object))
    _pydoc.describe = _scipy_pydoc_describe

    import inspect as _inspect
    _old_inspect_getfile = _inspect.getfile
    def _scipy_inspect_getfile(object):
        _old_inspect_getfile.__doc__
        if isinstance(object,_ModuleLoader):
            return object.__dict__['__file__']
        return _old_inspect_getfile(object)
    _inspect.getfile = _scipy_inspect_getfile



"""
Statistical Functions
=====================

This module contains a large number of probability distributions as
well as a growing library of statistical functions.

Each included distribution is an instance of the class rv_continous.
For each given name the following methods are available.  See docstring for
rv_continuous for more information

rvs   --  random variates with the distribution 
pdf   --  probability density function
cdf   --  cummulative distribution function
sf    --  survival function (1.0 - cdf)
ppf   --  percent-point function (inverse of cdf)
isf   --  inverse survival function
stats --  mean, variance, and optionally skew and kurtosis

Calling the instance as a function returns a frozen pdf whose shape,
location, and scale parameters are fixed. 

The distributions available with the above methods are:

CONTINUOUS  (Total == 81 distributions)
===========
norm            --  Normal (Gaussian)
alpha           --  Alpha
anglit          --  Anglit
arcsine         --  Arcsine
beta            --  Beta
betaprime       --  Beta Prime
bradford        --  Bradford
burr            --  Burr
fisk            --  Fisk
cauchy          --  Cauchy
chi             --  Chi
chi2            --  Chi-squared
cosine          --  Cosine
dgamma          --  Double Gamma
dweibull        --  Double Weibull
erlang          --  Erlang
expon           --  Exponential
exponweib       --  Exponentiated Weibull
exponpow        --  Exponential Power
fatiguelife     --  Fatigue Life (Birnbaum-Sanders)
foldcauchy      --  Folded Cauchy
f               --  F (Snecdor F)
foldnorm        --  Folded Normal
frechet_r       --  Frechet Right Sided, Extreme Value Type II (Extreme LB) or weibull_min
frechet_l       --  Frechet Left Sided, Weibull_max
genlogistic     --  Generalized Logistic
genpareto       --  Generalized Pareto
genexpon        --  Generalized Exponential
genextreme      --  Generalized Extreme Value
gausshyper      --  Gauss Hypergeometric
gamma           --  Gamma 
gengamma        --  Generalized gamma
genhalflogistic --  Generalized Half Logistic
gompertz        --  Gompertz (Truncated Gumbel)
gumbel_r        --  Right Sided Gumbel, Log-Weibull, Fisher-Tippett, Extreme Value Type I
gumbel_l        --  Left Sided Gumbel, etc.
halfcauchy      --  Half Cauchy
halflogistic    --  Half Logistic
halfnorm        --  Half Normal
hypsecant       --  Hyperbolic Secant
invgamma        --  Inverse Gamma
invnorm         --  Inverse Normal
invweibull      --  Inverse Weibull
johnsonsb       --  Johnson SB
johnsonsu       --  Johnson SU
laplace         --  Laplace
logistic        --  Logistic
loggamma        --  Log-Gamma
loglaplace      --  Log-Laplace (Log Double Exponential)
lognorm         --  Log-Normal
gilbrat         --  Gilbrat
lomax           --  Lomax (Pareto of the second kind)
maxwell         --  Maxwell
mielke          --  Mielke's Beta-Kappa
nakagami        --  Nakagami
ncx2            --  Non-central chi-squared
ncf             --  Non-central F
t               --  Student's T
nct             --  Non-central Student's T
pareto          --  Pareto
powerlaw        --  Power-function
powerlognorm    --  Power log normal
powernorm       --  Power normal
rdist           --  R distribution
reciprocal      --  Reciprocal
rayleigh        --  Rayleigh
rice            --  Rice
recipinvgauss   --  Reciprocal Inverse Gaussian
semicircular    --  Semicircular
triang          --  Triangular
truncexpon      --  Truncated Exponential
truncnorm       --  Truncated Normal
tukeylambda     --  Tukey-Lambda
uniform         --  Uniform
von_mises       --  Von-Mises (Circular)
wald            --  Wald
weibull_min     --  Minimum Weibull (see Frechet)
weibull_max     --  Maximum Weibull (see Frechet)
wrapcauchy      --  Wrapped Cauchy
ksone           --  Kolmogorov-Smirnov one-sided (no stats)
kstwobign       --  Kolmogorov-Smirnov two-sided test for Large N (no stats)


DISCRETE    (Total == 10 distributions)
============
binom           --  Binomial
bernoulli       --  Bernoulli
nbinom          --  Negative Binomial
geom            --  Geometric
hypergeom       --  Hypergeometric
logser          --  Logarithmic (Log-Series, Series)
poisson         --  Poisson
randint         --  Discrete Uniform
zipf            --  Zipf
dlaplace        --  Discrete Laplacian

Statistical Functions (adapted from Gary Strangman)

gmean
hmean
mean
cmedian
median
mode
tmean
tvar
tmin
tmax
tstd
tsem
moment
variation
skew
kurtosis
describe
skewtest
kurtosistest
normaltest

itemfreq
scoreatpercentile
percentileofscore
histogram2
histogram
cumfreq
relfreq

obrientransform
samplevar
samplestd
signaltonoise
var
std
stderr
sem
z
zs
zmap

threshold
trimboth
trim1
cov
corrcoef

f_oneway
paired
pearsonr
spearmanr
pointbiserialr
kendalltau
linregress

ttest_1samp
ttest_ind
ttest_rel
kstest
chisquare
ks_2samp
meanwhitneyu
tiecorrect
ranksums
wilcoxon
kruskal
friedmanchisquare

ansari
bartlett
levene
shapiro
anderson
binom_test
fligner
mood
oneway


glm
anova

Plot-tests

probplot
ppcc_max
ppcc_plot

For many more stat related functions install the software R and the 
interface package rpy.
"""

postpone_import = 1
global_symbols = ['mean','median','std','cov','corrcoef','find_repeats']



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from lines import *
from polymap import *
from cellarray import *
from graph2d import *

def paws ( ) :
    i = raw_input ("Type in any string to continue; ^C to return to prompt. ")
    return

def demo () :
   x0 = arange(50, typecode = Float)
   y0 = zeros(50, Float)
   x1 = 49 * ones(50, Float)
   y1 = arange(50, typecode = Float)
   ly = Lines (x0 = x0,y0 = y0,x1 = x1,y1 = y1)
   g0 = Graph2d ( ly , titles = "Just Lines")
   g0.plot ()
   paws ()
   ly.set (color = "red")
   g0.change (titles = "Lines colored red")
   g0.plot ()
   paws ()
   ly.set (width=4.0,type="dashdotdot")
   g0.change(titles = "Wide lines, dashdotdot style")
   g0.plot ()
   paws ()
   n = [4,3,3,3,3,4,4,3,3,3,3]
   x = [0.,1.,1.,0.,0.,1.5,1.,1.5,3.,0.,1.5,3.,2.,1.5,2.,1.,2.,3.,3.,2.,
        1.,2.,2.,1.,2.,3.,1.5,1.,2.,1.5,1.,1.5,0.,0.,3.,1.5]
   y = [1.,2.,7.,8.,1.,1.,2.,0.,1.,1.,1.,1.,2.,1.,2.,2.,2.,1.,8.,7.,
        2.,2.,7.,7.,7.,8.,8.,7.,7.,8.,7.,8.,8.,8.,8.,9.]
   z = [2.5,1.2,1.5,1.2,.5,2.5,2.,1.2,.5,1.2,1.5]
   p1 = Polymap(x=x,y=y,z=z,n=n)
   g0.replace(1,p1)
   g0.change(titles = "Stained glass window")
   g0.plot ()
   paws ()
   p1.set(z = array("AMZMmAzMmMZ",'b') )
   g0.plot ()
   paws ()
   nx= 10
   ny= 19
   # ndx = zeros ( (nx, ny ), Int)
   # for jj in range (ny) :
   #    for ii in range (nx) :
   #        ndx [ii, jj] = ii + (jj - 1) * nx
   ndx = reshape (arange (nx * ny, typecode = 'b'), (nx, ny))
   cla = CellArray ( z = ndx )
   gca = Graph2d ( cla , titles = "Cell Array", axis_scales = "linlin" )
   gca.plot ( )
   paws ( )
   # -------------------- generate sombrero function
   nz = 20
   x = arange (-nz, nz+1, typecode = Float )
   y = x
   z = zeros ((2*nz + 2, 2*nz + 2), Float)
   for i in range ( len (x) ) :
      for j in range ( len (y)) :
         r = sqrt ( x [i] * x[i] + y [j] * y [j] ) + 1.e-12
         z [i, j] = sin (r) / r
   # cell array plot
   cla.set (z = z)
   gca.change (titles = "Sombrero Function",axis_limits="defaults")
   gca.plot ( )
   paws ( )

print "Type demo() to proceed."



from __future__ import nested_scopes

import string, time, array
import rand
import Numeric
import sys
import math
from types import *
import types
Num = Numeric
import scipy.special
special = scipy.special
from scipy_base.fastumath import *
from scipy_base import vectorize
acos = arccos

SequenceType = [types.TupleType, types.ListType, array.ArrayType, Num.ArrayType]

def _check_shape(sh):
   if type(sh) not in SequenceType:
      sh = [sh]
   for val in sh:
      if not isinstance(val,types.IntType):
         raise ValueError, "Each element of the shape parameter must be an integer."
   prod = Num.product(sh)
   return tuple(sh), prod


ArgumentError = "ArgumentError"

def multivariate_normal(mean, cov, size=None):
       """returns an array containing multivariate normally distributed random
          numbers with specified mean and covariance.

          mean must be a 1 dimensional array. cov must be a square two dimensional
          array with the same number of rows and columns as mean has elements.

          The first form returns a single 1-D array containing a multivariate
          normal.

          The second form returns an array of shape (m, n, ..., cov.shape[0]).
          In this case, output[i,j,...,:] is a 1-D array containing a multivariate
          normal."""
       if isinstance(size, IntType): 
           size = [size]
       if size is None:
           n = 1
       else:
           n = Num.product(size)
       output = rand.multivariate_normal(mean, cov, n)
       if size is not None:
           final_shape = list(size[:])
           final_shape.append(mean.shape[0])
           output.shape = final_shape
       return output       
    
#####################################
# General purpose continuous
######################################

def randwppf(ppf, args=(), size=None):
    """returns an array of randomly distributed integers of a distribution
    whose percent point function (inverse of the CDF) is given.

    args is a tuple of extra arguments to the ppf function (i.e. shape,
    location, scale), and size is the size of the output.  Note the ppf
    function must accept an array of q values to compute over.
    """
    U = random(size=size)
    return apply(ppf, (U,)+args)

def randwcdf(cdf, mean=1.0, args=(), size=None):
    """returns an array of randomly distributed integers of a distribution
    whose cumulative distribution function (CDF) is given.

    mean is the mean of the distribution (helps the solver).
    args is a tuple of extra arguments to the cdf function (i.e. shape,
    location, scale), and size is the size of the output.  Note the
    cdf function needs to accept a single value to compute over.
    """
    import scipy.optimize as optimize
    def _ppfopt(x, q, *nargs):
        newargs = (x,)+nargs
        return cdf(*newargs) - q

    def _ppf(q, *nargs):
        return optimize.fsolve(_ppfopt, mean, args=(q,)+nargs)

    _vppf = vectorize(_ppf)
    U = random(size=size)
    return apply(_vppf,(U,)+args)


#################################################
## DISCRETE
##################################################

def multinom(trials, probs, size=None):
    """returns array of multinomial distributed integer vectors.

           trials is the number of trials in each multinomial distribution.
           probs is a one dimensional array. There are len(prob)+1 events. 
           prob[i] is the probability of the i-th event, 0<=i<len(prob).
           The probability of event len(prob) is 1.-Numeric.sum(prob).

       The first form returns a single 1-D array containing one multinomially
           distributed vector.

           The second form returns an array of size (m, n, ..., len(probs)).
           In this case, output[i,j,...,:] is a 1-D array containing a multinomially
           distributed integer 1-D array."""
        # Check preconditions on arguments
    probs = Num.array(probs)
    if len(probs.shape) != 1:
        raise ArgumentError, "probs must be 1 dimensional."
        # Compute shape of output
    if type(size) == type(0): size = [size]
    final_shape = size[:]
    final_shape.append(probs.shape[0]+1)
    x = rand.multinomial(trials, probs.astype(Num.Float32), Num.multiply.reduce(size))
        # Change its shape to the desire one
    x.shape = final_shape
    return x


##############################
## Functions from old rv2.py
##############################


class _pranv:
   """Encapsulating class for all random number generator methods and data

   All <_pranv> data and method attributes are private.  The single
   instance <_inst> is created and public aliases to external methods are
   provided at the end of the module.  To get the alias, delete the leading
   underscore from the name of the corresponding method in class <_pranv>.
   There are no public data attributes.  This class should have only one
   instance, <_inst>."""

#                    Internal (private) data attributes:
# Line
# Number
#     _algorithm_name   one-line description of uniform(0,1) algorithm used
#     _count            number of delivered random()s; Python long integer
#     _index            integer index to next uniform(0,1) in ranbuf[]
#     _iterator         a tuple of tuples used by some uniform generators
#     _ln_fac           a double array containing ln(n!) for n = 0, ..., 129
#     _ranbuf           a double array to store batched uniform(0,1) randoms
#     _ranbuf_size      integer, length of _ranbuf
#     _fillbuf          a function variable, aliasing a uniform(0,1) generator
#     _seed             initial Python long integer from user or clock
#     _series           array of pseudo-random series values, or single value
#     _second_nrv       Normal pseudo-randoms are produced in pairs; 2nd here.

#                    Internal (private) utility methods:

#     __init__()            Initialize <_pranv> instance _inst; called once.
#     _build_iterator()     Construct _iterator; allocate _series, _ranbuf.
#     _cmrg(size=None)    Fill ranbuf with random #s from algorithm 'cmrg'.
#     _fill_ln_fac()        Calculate and store ln(n!) for n = 0 through 129.
#     _flip(size=None)    Fill ranbuf with randoms from algorithm 'flip'.
#     _ln_factorial(n)      Return ln(n!), from ln_fac[n] or by calculation.
#     _smrg(size=None)    Fill ranbuf with random #s from algorithm 'sMRG'.
#     _twister(size=None) Fill ranbuf with randoms from algorithm 'twister'.

#            External (private, but aliased public) utility methods:

#     _initial_seed()        Return the seed used to start this random series.
#     _initialize(file_string=None, algorithm='cmrg', seed=0L)        Restart.
#     _random_algorithm()    Return a 1-line description of current generator.
#     _random_count()    Return number of randoms generated in current series.
#     _random(size=None)             Return next pseudo-random uniform(0,1).
#     _save_state(file_string='prvstate.dat')      Save <_inst> state to disk.

#  External (private, but aliased public) non-uniform random variate methods:

#     _choice(seq=(0,1), size=None)       <seq> item
#     _geom(pr_failure=0.5,
#                              size=None) integer     non-negative
#     _hypergeom(tot=35, good=25,
#                   sample=10, size=None) integer     >= max(0, sample-good)
#                                                       <= min(sample, bad)
#     _logser(p=0.5, size=None)      integer     positive
#     _von_Mises(mode=0.0, shape=1.0,
#                              size=None) double      > -pi, < +pi
#     _Wald(mean=1.0, scale=1.0,
#                              size=None) double      positive
#     _Zipf(a=4.0, size=None)             integer     positive

#           External (private, but aliased public) geometrical,
#                permutation, and subsampling routines:

#     _in_simplex(mseq=5*[0.0], bound=1.0)            Return point in simplex.
#     _in_sphere(center=5*[0.0], radius=1.0)           Return point in sphere.
#     _on_simplex(mseq=5*[0.0], bound=1.0)            Return point on simplex.
#     _on_sphere(center=5*[0.0], radius=1.0)           Return point on sphere.
#     _sample(inlist=[0,1,2], sample_size=2)      Return simple random sample.
#     _smart_sample(inlist=[0,1,2], sample_size=2)    Return 'no-dups' sample.


   def __init__(self):
      """Initialize the single class <_pranv> instance.

      """
#  Declaration of class _pranv data attributes, all introduced here mostly for
#  documentation purposes.  Most are allocated or set in _initialize().

      self._algorithm_name = '' # one-line aLgorithm description
      self._count = 0L     #  number of pseudorandoms generated in this series
      self._index = 0      #  index to output buffer array ranbuf[]
      self._iterator = ((),)  # tuple of tuples used by 'flip' and 'twister'.
      self._ln_fac = array.array('d', 130*[0.0])    # ln(n!); n = 0 to 129;
                           #  double array used by some discrete generators.
      self._ranbuf = []    #  uniform(0,1) output buffer array; allocated in
                           #  _build_iterator(), called by _initialize().
      self._ranbuf_size = 0    #  This is just len(_ranbuf), after allocation.
      self._fillbuf = None #  function variable pointing to uniform(0,1) gen.
      self._seed = 0L      #  starting seed from user or system clock
      self._series = []    #  rotating array of random integers, longs or
                           #  doubles; allocated in _build_iterator.
      self._second_nrv = None  # 2nd of generated pair of normal random vars.
#            End of class <_pranv> data attribute declarations

      self._fill_ln_fac()  # Calculate values of  _ln_fac[n] = ln(n!)
                           # for n = 0 to 129; only need to do this once.
      self._initialize()   # Do detailed initialization.  <_initialize()>
                           # may also be called by the user (via its global
                           # alias that drops the leading '_') to change
                           # generators.

   def _build_iterator(self):
      """Construct <_iterator[]> if necessary; allocate _ranbuf and _series.

      _build_iterator()

      'flip' and 'twister', pseudo-random integer generators which manipulate
      long series of integers, utilize a tuple of tuples to simplify and speed
      up the logic.  This function constructs the (global) tuple of tuples,
      <_iterator>, if it's not already there.  The array <_ranbuf> is
      allocated if needed and its length, <_ranbuf_size>, is set.  Also, the
      array <_series> is allocated if necessary.  <_build_iterator> is an
      internal (private) <_pranv> method, called whenever the selected
      uniform(0,1) pseudo-random generator changes, by <_initialize()>."""

      if self._fillbuf == self._cmrg:
         if self._ranbuf_size != 660:    # 'cmrg' does not use _iterator[].
            del self._ranbuf
            self._ranbuf = array.array('d', 660*[0.0])     # output buffer
            self._ranbuf_size = 660
         if len(self._series) != 6:
            del self._series
            self._series = array.array('d', 6*[0.0])

      elif self._fillbuf == self._flip:
         if len(self._iterator) != 55:  # May need to recreate _ranbuf ,
            del self._series            # _iterator and _series.
            self._series = array.array('l',56*[0])
            del self._iterator
            self._iterator = 55 * [()]
            i = 1                  # The 'flip' _iterator is:
            j = 32                 # ( (1,32), (2,33), ..., (24,55), (25,1),
            while j < 56:          #    (26,2), ..., (55,31) ).  Note that
               self._iterator[i-1] = (i, j) # _iterator is global.  A list
               i = i + 1           # here, _iterator converts to tuple below.
               j = j + 1

            j = 1
            while j < 32:
               self._iterator[i-1] = (i, j)
               i = i + 1
               j = j + 1

            self._iterator = tuple(self._iterator)

         if self._ranbuf_size != 660:
            del self._ranbuf
            self._ranbuf = array.array('d', 660*[0.0])  # output buffer
            self._ranbuf_size = 660

      elif self._fillbuf == self._smrg:
         if self._ranbuf_size != 660:    # 'SMRG' does not use _iterator[].
            del self._ranbuf
            self._ranbuf = array.array('d', 660*[0.0])  # output buffer
            self._ranbuf_size = 660

         del self._series
         self._series = 0L               # _series is a single Python long.

      elif self._fillbuf == self._twister:
          if self._ranbuf_size != 624:   # May need to reload _ranbuf,
            del self._ranbuf             # _series and _iterator.
            self._ranbuf = array.array('d', 624*[0.0])
            self._ranbuf_size = 624
            del self._series
            self._series = array.array('L', 624*[0L])
            del self._iterator
            self._iterator = 624 * [()]
                                   #  _iterator is:
            k = 0                  # ( (0,1,397),(1,2,398),...,(226,227,623),
            while k < 227:         # (227,228,0),228,229,1),...,(622,623,395),
               self._iterator[k] =  (k, k + 1, k + 397) # (623,0,386) ).A list
               k = k + 1           # here, _iterator converts to tuple below.

            while k < 623:
               self._iterator[k] =  (k, k + 1, k - 227)
               k = k + 1

            self._iterator[623] =   (623, 0, 396)
            self._iterator = tuple(self._iterator)

      else:
         pass                   # Can't get here.

   def _cmrg(self):
      """Produce batch of uniform(0,1) RVs from L'Ecuyer's MRG32k3a generator

      _cmrg()

      See L'Ecuyer, Pierre., "Good parameters and implementations for combined
      multiple recursive random number generators," May, 1998, to appear in
      "Operations Research."  To download a Postscript copy of the article:
          http://www.iro.umontreal.ca/~lecuyer/myftp/papers/combmrg2.ps
      The period is about 2**191, (or 10**57), and it is well-behaved in all
      dimensions through 45. The generator has been tested extensively by
      L'Ecuyer; no statistical faults were found."""

    # This algorithm is much simpler than the implementation appearing below,
    # which has been hand-optimized to minimize array references. The savings
    # in execution time was about 25%.  Here is the underlying algorithm:
    #
    #     s = self.series                         # local alias
    #
    #     p1 = 1403580.0 * s[1] - 810728.0 * s[0]
    #     k =         floor(p1  /  4294967087.0)  # first modulus, 2**32 - 209
    #     p1 =        p1 - k    *  4294967087.0
    #     if p1 < 0.0: p1 = p1  +  4294967087.0
    #
    #     p2 = 527612.0 * s[5] - 1370589.0 * s[3]
    #     k =         floor(p2  /  4294944443.0)  # 2nd modulus, 2**32 - 22853
    #     p2 =          p2 - k  *  4294944443.0
    #     if p2 < 0.0: p2 = p2  +  4294944443.0
    #     s[3] = s[4]
    #     s[4] = s[5]
    #
    #     dif = p1 - p2
    #     if dif <= 0.0: return (dif + 4294967087 * 2.328306549295728e-10
    #     else:          return               dif * 2.328306549295728e-10
    #

      ranlim = self._ranbuf_size                  # local alias
      buf = self._ranbuf                          # local alias
      s = self._series                            # local alias
      (s0, s1, s2, s3, s4, s5) = s                # Unpack to local aliases.
      i = 0
      while i < ranlim:
         p11 = 1403580.0 * s1 - 810728.0 * s0
         p11 = p11  -   floor(p11 / 4294967087.0) * 4294967087.0
         if p11 < 0.0: p11 = p11 +  4294967087.0  # first mod, 2**32-209

         p2 = 527612.0 * s5 - 1370589.0 * s3
         p2 = p2  -  floor(p2 / 4294944443.0) * 4294944443.0
         if p2 < 0.0: p2 = p2 + 4294944443.0      # 2nd mod, 2**32-22853

         s0 = p11
         s3 = p2
         dif = p11 - p2
         if dif < 0: dif = dif + 4294967087.0
         buf[i] = dif * 2.328306549295728e-10
         i = i + 1

         p1 = 1403580.0 * s2 - 810728.0 * s1
         p1 = p1  -  floor(p1 / 4294967087.0) * 4294967087.0
         if p1 < 0.0: p1 = p1 + 4294967087.0      # first mod, 2**32-209

         p2 = 527612.0 * p2 - 1370589.0 * s4
         p2 = p2  -  floor(p2 / 4294944443.0) * 4294944443.0
         if p2 < 0.0: p2 = p2 + 4294944443.0      # 2nd mod, 2**32-22853

         s1 = p1
         s4 = p2
         dif = p1 - p2
         if dif < 0: dif = dif + 4294967087.0
         buf[i] = dif * 2.328306549295728e-10
         i = i + 1

         p1 = 1403580.0 * p11 - 810728.0 * s2
         p1 = p1  -  floor(p1 / 4294967087.0) * 4294967087.0
         if p1 < 0.0: p1 = p1 + 4294967087.0      # first mod, 2**32-209

         p2 = 527612.0 * p2 - 1370589.0 * s5
         p2 = p2  -  floor(p2 / 4294944443.0) * 4294944443.0
         if p2 < 0.0: p2 = p2 + 4294944443.0      # 2nd mod, 2**32-22853

         s2 = p1
         s5 = p2
         dif = p1 - p2
         if dif < 0: dif = dif + 4294967087.0
         buf[i] = dif * 2.328306549295728e-10
         i = i + 1

      s[0] = s0                                # Save the coefficients
      s[1] = s1                                # for the next batch of
      s[2] = s2                                # 660 uniform randoms.
      s[3] = s3                                # s = [s0,s1,s2,s3,s4,s5]
      s[4] = s4                                # doesn't work here because
      s[5] = s5                                # s is an array, not a list.
      self._index = 0
      self._count = self._count + ranlim


   def _fill_ln_fac(self):
      """Calculate and store array values of  <_ln_fac[n]> = ln(n!).

      _fill_ln_fac()

      This internal routine is called only once, by  __init__() in
      class <_pranv> during the instantiation of <_inst>."""
      self._ln_fac[0] = 0.0
      i = 0
      sum = 0.0
      while i < 129:
         i = i + 1
         sum = sum + log( float(i) )
         self._ln_fac[i] = sum


   def _flip(self):
      """Produce a batch of uniform(0,1) RVs with Knuth's 1993 generator.

      _flip()

      See Knuth, D.E., "The Stanford GraphBase: A Platform for Combinatorial
      Computing," ACM Press, Addison-Wesley, 1993, pp 216-221.  The period for
      the underlying pseudo-random integer series is at least 3.6e16, and may
      be nearly 4e25. The low-order bits of the integers are just as random
      as the high-order bits. The requirements are 32-bit integers and two's
      complement arithmetic.  The recurrence is quite fast, but for sensitive
      applications, autocorrelations or other statistical defects may dictate
      the use of a more complex generator, a longer series or both."""
      ranlim = self._ranbuf_size                  # local alias
      buf = self._ranbuf                          # local alias
      s = self._series                            # local alias
            # Here is the algorithm, C-language style:
            #    ii = 1             # Refresh series of 55 random integers.
            #    jj = 32
            #    while jj < 56:     # Calculate (s[ii] - s[ii+31]) % 2**31.
            #       s[ii] = (s[ii] - s[jj]) & 0x7fffffff
            #       ii = ii + 1
            #       jj = jj + 1
            #    jj = 1
            #    while jj < 32:     # Calculate (s[ii] - s[ii-24]) % 2**31.
            #       s[ii] = (s[ii] - s[jj]) & 0x7fffffff
            #       ii = ii + 1
            #       jj = jj + 1
            #
            # And here it is using <_iterator>:

      i = 0
      while i < ranlim:
         for (ii, jj) in self._iterator:
            s[ii] = (s[ii] - s[jj]) & 0x7fffffff
            buf[i] = ( float(s[ii]) + 1.0 ) * 4.6566128709089882e-10
            # The constant is 1/(2**31+1), so stored value is in (0,1).
            i = i + 1

      self._index = 0
      self._count = self._count + ranlim


   def _ln_factorial(self, n=0.0):
      """Return ln(n!)from the array <_ln_fac[n]> or use Stirling's formula.

      _ln_factorial(n=0.0)

      For 0 <= n <= 129, the value pre-calculated and stored in <_ln_fac[]>
      by internal routine <_fill_ln_fac()> is returned.  For n >= 130, the
      Stirling asymptotic approximation with 3 terms is used.
      Approximation error is negligible.  NOTE:  n must be non-negative
      and should be integral.  There is no error-checking in this function
      because it is internal, and we don't make mistakes inside this class."""

      if n < 130:
         return self._ln_fac[int(n)]
      else:
         y = n + 1.0
         yi = 1.0 / y
         yi2 = yi * yi
         return ((((  0.793650793650793650e-3) * yi2  #  1 / (1260 * (n+1)**5)
                    - 0.277777777777777778e-2) * yi2  # -1 / ( 360 * (n+1)**3)
                    + 0.833333333333333333e-1) * yi   # +1 / ( 8   * (n+1)   )
                    + 0.918938533204672742            # +(log(sqrt(2*math.pi))
                    + (y - 0.5) * log(y) - y )

   def _smrg(self):
      """Produce a batch of uniform(0,1) RVs from Wu's mod 2**61 - 1 generator

      _smrg()

      See Wu, Pei-Chi, "Multiplicative, Congruential Random-Number Generators
      with multiplier (+ or -) 2**k1 (+ or -) 2**k2 and Modulus 2**p - 1, ACM
      Transactions on mathematical Software, June, 1997, Vol. 23, No. 2,
      pp 255 - 265.  The generator has modulus 2**61 - 1, which is the
      Mersenne prime immediately following 2**31 - 1, and multiplier
      37**458191 % (2**61 - 1). Because 37 is the minimal primitive root of
      2**61 -1, the generator has full period (2**61 -2, about 2.3e18).
      It was found by Wu to have the best performance on spectral tests
      (through dimension 8) of any multiplier of the type 37**k % (2**61 - 1)
      with k in the range from 1 to 1,000,000.
      Generated pseudo-random numbers range from 4.337e-19 to 1.0 - 2**53
      -- all bits are random, an advantage over 31- or 32-bit generators. """
      buf = self._ranbuf                          # local alias
      s = self._series                            # local alias
      ranlim = self._ranbuf_size                  # local alias
      for i in xrange(ranlim):
         s = s * 2137866620694229420L % 0x1fffffffffffffffL  # The first
                                 # constant is 37**458191 % (2**61 - 1),
                                 # and the second is 2**61-1.
         buf[i] = float(s) * 4.3368086899420168e-19 # For s = 2**61 - 2,
                                                    # this is 1 - 2**-53.
      self._index = 0
      self._series = s           # save the current seed in <_series>
      self._count = self._count + ranlim

   def _twister(self):
      """Produce a batch of uniform(0,1) RVs from the Mersenne Twister MT19937

      _twister()

      See M. matsumoto and T. Nishamura, "Mersenne Twister," ACM Transactions
      on Modeling and Computer Simulation, Jan. 1998, vol. 8, no. 1, pp 3-30.
      The period is 2**19937 - 1,(> 1e6000); the generator has a 623 dim-
      ensional equi-distributional property.  This means that every sequence
      of less than 624 pseudo-random 32-bit integers occurs equally often. It
      has passed a number of statistical tests, including those in Marsaglia's
      "Diehard" package. """
      buf = self._ranbuf                          # local alias
      s = self._series                            # local alias
    # Here is the first part of the algorithm, C-language style:
    # if i == 624:             # self._index points to long ints in _series[].
    #    k = 0                 # Generate 624 new pseudo-random long integers.
    #    while k < 227:        # 227 = 624 - 397
    #       y = (s[ k ] & 0x80000000L) | (s[k + 1] & 0x7fffffffL)
    #       s[k] = s[k + 397] ^ (y >> 1) ^ (y & 0x1) * 0x9908b0dfL
    #       k = k + 1
    #
    #    while k < 623:
    #       y = (s[ k ] & 0x80000000L) | (s[k + 1] & 0x7fffffffL)
    #       s[k] = s[k - 227] ^ (y >> 1) ^ (y & 0x1) * 0x9908b0dfL
    #       k = k + 1
    #
    #    y =    (s[623] & 0x80000000L) | (s[  0  ] & 0x7fffffffL)
    #     s[623] =  s[  396  ] ^ (y >> 1) ^ (y & 0x1) * 0x9908b0dfL
    #
    # And here is is in Python, using the global iterator, just three lines:
      for (k, kp1, koff) in self._iterator:
         y = (s[k] & 0x80000000L) | (s[kp1] & 0x7fffffffL)
         y = s[koff] ^ (y >>1) ^ (y & 0x1) * 0x9908b0dfL
         s[k] = y
         y = y ^ (y >> 11)
         y = y ^ (y <<  7) & 0x9d2c5680L
         y = y ^ (y << 15) & 0xefc60000L
         y = y ^ (y >> 18)
         buf[k] = (float(y) + 1.0) * 2.3283064359965952e-10
                                 # The constant is 1.0 / (2.0**32 +1.0).
      self._index = 0
      self._count = self._count + self._ranbuf_size

#            External (private; aliased public) utility methods:

   def _initialize(self, file_string=None, algorithm='cmrg', seed=0L):
      """Set uniform algorithm and seed, or restore state from disk.

      Called by <__init__()> or by user, via the alias <initialize()>.
      if <file_string> is specified, it can be a null string, indicating
      that class _pranv state is not to be retrieved from a file; a single
      blank or the string 'default', indicating that _pranv state is to be
      restored from the file with the default file name ('prvstate.dat'); or
      the path/file name of a file containing a _pranv state.  <algorithm> may
      be 'cmrg', 'flip', 'smrg', or 'twister'; see the documentation for
      guidance.  <seed> is by default 0, indicating the system clock is to
      be used to determine algorithm starting value(s), or a Python long
      integer.  For some of the algorithms, a negative seed has a special
      meaning; see the code below.  <algorithm> and <seed are ignored if
      <file_string is non-null."""
      if file_string:                 # User wants to restore generator state.
         if (file_string == ' ' or    # a space (not the null string)
          string.strip( string.lower(file_string) ) == 'default'):
            file_string = 'prvstate.dat'
         f = open(file_string, 'r')   # Get saved state of uniform generator.
         inlist = f.readlines()
         f.close()
         self._algorithm_name = inlist[1]    # Skip the header (inlist[0]).
         self._algorithm_name = self._algorithm_name[:-1]  # Drop the \n.
         self._seed = eval(inlist[2])
         self._count = eval(inlist[3])
         self._second_nrv = eval(inlist[4])
         self._index = eval(inlist[5])

         pos = string.find(self._algorithm_name, ":")
         if algorithm == 'cmrg':
            self._fillbuf = self._cmrg       # Set generator function pointer.

         elif algorithm == 'flip':
            self._fillbuf = self._flip

         elif algorithm == 'smrg':
            self._fillbuf = self._smrg

         elif algorithm == 'twister':
            self._fillbuf = self._twister

         else:
            pass                     # can't get here

         self._build_iterator()      # Construct  _iterator, and allocate
                                     # _series and _ranbuf if necessary.
         if type(self._series) == array.ArrayType:
            offset = 6
            for i in xrange( len(self._series) ):         # 'CMRG', 'Flip', or
               self._series[i] = eval(inlist[offset + i]) # 'Twister'

            offset = len(self._series) + 6
            for i in xrange(self._ranbuf_size):
               self._ranbuf[i] = eval(inlist[offset + i])

         else:
            self._series = eval( inlist[6] )              # 'SMRG'
            offset = 7
            for i in xrange(self._ranbuf_size):
               self._ranbuf[i] = eval(inlist[offset + i])

      else:                            # User wants a new generator.
         self._second_nrv = None       # Initialize storage for 2nd normal RV.
         self._count = 0L              # Initialize random() delivery count.
         algorithm = string.strip( string.lower(algorithm) )
         if algorithm == 'cmrg':
            self._algorithm_name = \
        "'CMRG': Combined Multiplicative Recursion  MRG32k3a (L'Ecuyer, 1998)"
            self._fillbuf = self._cmrg # Set generator function pointer.
            self._build_iterator()     # Allocate _series and _ranbuf.
            if seed < 0:
               self._seed = long(seed)                # User wants all initial
               seed = float((-seed) & 0x7fffffffL)    # values equal.
               for i in range(6): self._series[i] = seed

            else:
               if seed == 0:          # Generate initial seed from epoch time.
                  t = (long(time.time()*128) * 1048576L) & 0x7fffffffffffffffL
                  seed = ((t & 0x7fffffffL)^(t >> 32)) & 0x7fffffffL # 31 bits
                  seed = max(seed, 1L) # <seed> must be positive in CMRG
                  seed = min(seed, 4294944442L)  # 2**32 - 22853 - 1

               self._seed = long(seed) # Save for user inquiry.
               self._series[0] = float(seed)
               for j in range(1,6):    # Use a standard linear multiplicative
                  k = seed / 127773    # congruential generator to get initial
                  seed = 16807 * (seed - k * 127773) - 2836 * k  # values for
                  if seed < 0: seed = seed + 0x7fffffff # other 5 multipliers.
                  self._series[j] = float(seed)

            self._index = self._ranbuf_size-1  # Request new batch of randoms.

         elif algorithm == 'flip':
            self._algorithm_name = \
               "'Flip': Subtractive Series Mod 2**31 (Knuth,1993)"
            self._fillbuf = self._flip  # Set generator function pointer.
            self._build_iterator()      # define_iterator;_ranbuf and _series
            if seed == 0L:              # Get seed from epoch time.
               t = (long(time.time() * 128) * 1048576L) & 0x7fffffffffffffffL
               seed = int( (t & 0x7fffffffL)^(t >> 32) ) & 0x7fffffffL # 31 b.
               seed = max(seed, 1)      # Insure seed is positive.

            self._seed = long(seed)  # Save for possible user inquiry.
            seed = seed & 0x7fffffff # Need positive 4-byte integer here.
            s = self._series         # local alias
            s[0] = -1                # <_series[0]> is a sentinel unused here.
            prev = seed              # <_series[]> contains signed integers.
            next = 1
            s[55] = prev
            i = 21
            while i:
               s[i] = next
               next = (prev - next) & 0x7fffffff     # difference mod 2**31
               if seed & 0x1: seed = 0x40000000 + (seed >> 1)
               else: seed = seed >> 1                # cyclic shift right
               next = (next - seed) & 0x7fffffff     # difference mod 2**31
               prev = s[i]
               i = (i + 21) % 55                     # Jump arround in array.

            self._index = self._ranbuf_size-1 # Request new batch of uniforms.
            self._random()                    # Exercize the generator by
            self._index = self._index + 219   # "using" 4 sets of 55 integers.

         elif algorithm == 'smrg':
            self._algorithm_name = \
               "'SMRG': Single Multiplicative Recursion m61-p3019 (Wu, 1997)"
            self._fillbuf = self._smrg       # Set generator function pointer.
            self._build_iterator()           # Allocate _ranbuf and _series
            if seed == 0L:
               t = (long( time.time() * 128 ) * 544510892281524561475242L)
               t = t & 0x3ffffffffffffffffffffffffffffffL  # Keep 122 bits.
               seed = ( t ^ (t >> 61) ) & 0x1fffffffffffffffL # 61 bits
                                 # The time() multiplier above is arbitrary
                                 # so long as t >= (but close to) 2**122.
            self._seed = seed    # Save for user inquiry.
            seed = abs(seed)     # Insure seed is positive.
            seed = min(seed, 2L**61 - 2)  # 2**61 - 1 is the modulus
            self._series = seed  #_series is not an array here; a Python long.
            self._index = self._ranbuf_size-1 # Request new batch of uniforms.
            self._random()       # Exercize the generator by "using" 16
            self._index = self._index + 15    # pseudo-random numbers.

         elif algorithm == 'twister':
            self._algorithm_name = \
         "'Twister': Mersenne Twister MT19937 (matsumoto and Nishamura, 1998)"
            self._fillbuf = self._twister # Set generator function pointer.
            self._build_iterator()        # Define _iterator;_ranbuf, _series.
            if seed == 0L:
                                          #  Construct a seed from epoch time.
                                          #  32-bit unsigned integers needed.
               t = (long(time.time() * 128) * 2097152L) & 0xffffffffffffffffL
               seed = ( (t & 0xffffffffL)^(t >> 32) ) & 0xffffffffL # 32 bits

            self._series[0] = seed   # <_series[]> contains unsigned integers.
            seed = abs(seed)
            self._seed = long(seed)  # Save the seed for later user inquiry.
            seed = seed & 0xffffffffL

            # Use an algorithm from Knuth(1981) to continue filling <_series>
            # with 623 more unsigned 32-bit pseudo-random integers.

            for k in xrange(1, 624):
               seed = (69069L * seed) & 0xffffffffL    # Retain lower 32 bits.
               self._series[k] = seed

            self._index = self._ranbuf_size - 1
                                # Force generation of first batch of 624
                                # pseudo-randoms.  <_index> points to the
                                # pseudo-random uniform just delivered from
                                # <_ranbuf>; none remain when <_index> is 623.

         else: raise ValueError, \
           "algorithm? --must be 'CMRG', 'Flip', 'SMRG', or 'Twister'"


   def _initial_seed(self):
      """Return seed for this series; was either user-supplied or from clock.

      initial_seed()

      The result is a Python long integer."""
      return self._seed

   def _random_algorithm(self):
      """Return 1-line description of uniform random number algorithm used.

      random_algorithm()

      The result is a string."""
      return self._algorithm_name

   def _random_count(self):
      """Return number of uniform(0,1) RVs used since the seed was last reset.

      random_count()

      A Python long integer is returned.  This the number of uniform(0,1)
      random numbers delivered to the user or consumed by _pranv methods."""
      return self._count - (self._ranbuf_size - 1 - self._index)


   def _save_state(self, file_string='prvstate.dat'):
      """Save <_pranv> state to disk for later recall and continuation.

      save_state(file_string='prvstate.dat')

      A backup file ('filename.bak' or 'prvstate.bak') is created if
      the file <file_string> can be opened, where <filename*> is
      the leading portion of <file_string> before the '.', if present."""

      try:
         f = open(file_string, 'r')     # If file exists, create backup file.
         temp = f.read()
         dot_pos = string.rfind(file_string,'.')
         if dot_pos == -1: bak_file_string = file_string + '.bak'
         else: bak_file_string = file_string[:dot_pos] + '.bak'
         f.close()
         f = open(bak_file_string, 'w') # Write backup file.
         f.write(temp)
         f.close()
      except IOError:                   # No file was found.
         pass
      f = open(file_string, 'w')        # Open file for current state.
                                        # Save state.  Arrays can't be
      outlist = []                      # pickled, so just write out the data.
      outlist.append('Python module rv 1.1 random numbers save_state: '
         + time.ctime( time.time() ) + '\n')  # save_state file header
      outlist.append(self._algorithm_name + '\n')
      outlist.append(`self._seed` + '\n')
      outlist.append(`self._count` + '\n')
      outlist.append(`self._second_nrv` + '\n')
      outlist.append(`self._index` + '\n')
      if type(self._series) is array.ArrayType:
         for i in xrange( len(self._series) ): # 'CMRG', 'Flip', and 'Twister'
            outlist.append(`self._series[i]` + '\n')

      else:                                          # In 'SMRG', _series is a
         outlist.append(`self._series` + '\n')       # single Python long.

      for i in xrange (self._ranbuf_size):     # buffer of uniform(0,1) RVs
            outlist.append(`self._ranbuf[i]` + '\n')

      f.writelines(outlist)
      f.close()

#             Uniform(0,1) Pseudo-random Variate Generator

   def _random(self, size=None):
      """Return a single random uniform(0,1), or <None> and fill buffer.

      """
      i = self._index                        # local alias
      if size is not None:
         size, Ns = _check_shape(size)
         buffer = Num.zeros(Ns,Num.Float)
         buf = self._ranbuf                  # local alias
         ranlim = self._ranbuf_size          # local alias
         for j in xrange( Ns ):
            i = i + 1
            if i == ranlim: self._fillbuf(); i = 0
            buffer[j] = buf[i]
         return Num.reshape(buffer, size)

      else:
         i = i + 1
         if i == self._ranbuf_size: self._fillbuf(); i = 0
         self._index = i
         return self._ranbuf[i]

      self._index = i       # Restore _ranbuf index and return (buffer filled).

   def _choice(self, seq=(0,1), size=None):
      """Return element k with probability 1/len(seq) from non-empty sequence.

      choice(seq=(0,1), size=None)

      Default is a 0, 1 coin flip.  <seq> must not be empty.  If <buffer> is
      specified, it must be a mutable sequence.  It is filled with random
      <seq> elements and <None> is returned.  Otherwise, a single <seq>
      element is returned."""
      lenseq = len(seq)
      if lenseq == 0:
         raise ValueError, '<seq> must not be empty'

      else:
         i = self._index                          # local alias
         if size is not None:
            size, Ns = _check_shape(size)
            buffer = Num.zeros(Ns,Num.Float)
            buflim = self._ranbuf_size            # local alias
            buf = self._ranbuf                    # local alias
            for j in xrange( Ns ):
               i = i + 1
               if i == buflim: self._fillbuf(); i = 0
               x = buf[i]
               buffer[j] = seq[int( floor(x * lenseq) )]
            return Num.reshape(buffer, size)

         else:
            i = i + 1
            if i == self._ranbuf_size: self._fillbuf(); i = 0
            x = self._ranbuf[i]
            self._index = i          # Restore updated value of _ranbuf index.
            return seq[int( floor(x * lenseq) )]

      self._index = i  # Update _ranbuf index and return (buffer[] is filled).

   def _geom(self, pr=0.5, size=None):
      """Return non-negative random integers from a geometric distribution.

      geom(pr=0.5, size=None)

      Z has the geometric distribution if it is the number of successes
      before the first failure in a series of independent Bernouli trials
      with probability of success 1 - <pr>.  0 <= pr < 1.
      The result is a non-negative integer less than or equal to 2**31 -1.
      If <buffer> is specified, it must be a mutable sequence.
      <buffer> is filled with geometric(pr) pseudo-randoms.
      Otherwise, a single geometric pseudo-random variate is returned."""

      pr = 1.0-pr    # Added to be consistent with distributions.py 
      if not 0.0 <= pr < 1.0:
         raise ValueError, '0.0 <=  <pr>  < 1.0'
      else:
         i = self._index                  # local alias
         buf = self._ranbuf               # local alias
         buflim = self._ranbuf_size       # local alias
         if size is not None:
            size, Ns = _check_shape(size)
            buffer = Num.zeros(Ns,Num.Float)
            out_len = Ns            
            j = 0

         else:
            out_len = 0
            j = -1

         if pr <= 0.9:            # <pr> is small or moderate;
            while j < out_len:            # use inverse transformation.
               pk = sum = 1.0 - pr
               successes = 0
               i = i + 1
               if i == buflim: self._fillbuf(); i = 0
               u = buf[i]
               while sum < u:
                  successes = successes + 1
                  pk = pk * pr
                  sum = sum + pk

               if out_len:
                  buffer[j] = successes
                  j = j + 1

               else:
                  self._index = i    # Restore updated value of _ranbuf index.
                  return successes

         else:                       # <pr> larger than 0.9.
            while j < out_len:
               i = i + 1
               if i == buflim: self._fillbuf(); i = 0
               u = buf[i]
               successes = floor( log(u) / log(pr) )
               successes = int( min(successes, 2147483647.0) )     # 2**31 - 1
               if out_len:
                  buffer[j] = successes
                  j = j + 1

               else:
                  self._index = i    # Restore updated value of _ranbuf index.
                  return successes

         self._index = i # Restore _ranbuf index and return (buffer[] filled).
         return Num.reshape(buffer, size)         

   def _hypergeom(self, tot=35, good=25, N=10, size=None):
      """Return hypergeometric pseudorandom variates: #"bad" in <sample>

      hypergeom(tot=35, good=25, N=10)

      Z has a hypergeometric distribution if it is the number of "bad"
      (tot-good) items in a sample of size <N> from a population of
      size tot items.  <tot>, <good> and <N> must be positive integers.  Also,
      <tot> >= <sample> >= 1.  The result  Z satisfies:
      max(0, <sample> - <good>) <= Z <= min(<sample>, <tot>-<good>).
      if size is not None is
      supplied, it must be a mutable sequence (list or array).  It is filled
      with hypergeometric pseudo-random integers and <None> is returned.
      Otherwise, a single hypergeometric pseudo-random integer is returned.
      See Fishman, "Monte Carlo," pp 218-221.  Algorithms HYP and HRUA* were
      used."""
      bad = tot - good
      alpha = int(bad); beta=int(good); n = int(N)
      if (alpha < 1) or (beta < 1) or (n < 1) or (alpha + beta) < n:
         raise ValueError, '<bad>, <good>, or <sample> out of range'

      else:
         i = self._index                  # local alias
         buf = self._ranbuf               # local alias
         buflim = self._ranbuf_size       # local alias
         if size is not None:
            size, Ns = _check_shape(size)
            buffer = Num.zeros(Ns,Num.Float)
            out_len = Ns            
            j = 0

         else:
            out_len = 0
            j = -1

         if n <= 10:                      # 10 is an empirical estimate.
            d1 = alpha + beta - n         # Use Fishman's HYP algorithm.
            d2 = float( min(alpha, beta) )
            while j < out_len:
               y = d2
               k = n
               while y > 0.0:
                  i = i + 1
                  if i == buflim: self._fillbuf(); i = 0
                  u = buf[i]
                  y = y - floor( u + y / (d1 + k) )
                  k = k - 1
                  if k == 0:
                     break

               z = int(d2 - y)
               if alpha > beta: z = n - z
               if out_len:
                  buffer[j] = z
                  j = j + 1

               else:
                  self._index = i
                  return z

         else:                            # Use Fishman's HRUA* algorithm.
            minalbe = min(alpha, beta)
            popsize = alpha + beta
            maxalbe = popsize - minalbe
            lnfac = self._ln_factorial    # function alias
            m = min(n, popsize - n)
            d1 = 1.715527770              # 2 * sqrt(2 / math.e)
            d2 = 0.898916162              # 3 - 2 * sqrt(3 / math.e)
            d4 = float(minalbe) / popsize
            d5 = 1.0 - d4
            d6 = m * d4 + 0.5
            d7 = sqrt((popsize - m) * n * d4 * d5 / (popsize - 1) + 0.5)
            d8 = d1 * d7 + d2
            d9 = floor( float((m + 1) * (minalbe + 1)) / (popsize + 2) )
            d10 = ( lnfac(d9) + lnfac(minalbe - d9) +
                    lnfac(m - d9) + lnfac(maxalbe - m + d9) )
            d11 = min( min(m, minalbe) + 1.0, floor(d6 + 7 * d7) )  # 7 for 9-
            while j < out_len:                               # digit precision
               while 1:                                      # in d1 and d2
                  i = i + 1
                  if i == buflim: self._fillbuf(); i = 0
                  x = buf[i]
                  i = i + 1
                  if i == buflim: self._fillbuf(); i = 0
                  y = buf[i]
                  w = d6 + d8 * (y - 0.5) / x
                  if w < 0.0 or w >= d11:
                     continue             # fast rejection; try another x, y

                  else:
                     z = floor(w)
                     t = d10 - ( lnfac(z) + lnfac(minalbe - z) +
                                 lnfac(m - z) + lnfac(maxalbe - m + z) )
                     if x * (4.0 - x) - 3.0 <= t:
                        break             # fast acceptance

                     else:
                        if x * (x - t) >= 1:
                           continue       # fast rejection, try another x, y

                        else:
                           if 2.0 * log(x) <= t:
                              break       # acceptance

               if alpha > beta:           # Error in HRUA*, this is correct.
                  z = m - z

               if m < n:                  # This fix allows n to exceed
                  z = alpha - z           # popsize / 2.

               if out_len:
                  buffer[j] = z
                  j = j + 1

               else:
                  self._index = i
                  return z

         self._index = i # Restore ranbuf index and return (buffer is filled).
         return Num.reshape(buffer, size)
      

   def _logser(self, pr=0.5, size=None):
      """Return logarithmic (log-series) positive pseudo-random integers;
      0<p<1.

      logser(pr=0.5, size=None)

      Z has the logseries(logarithmic)distribution if Pr{Z=i} = (a/i) * p ** i,
      for i = 1, 2, ... and a = -1.0 / log(1.0 - p). If <buffer> is specified,
      it must be a mutable sequence (list or array).  It is filled with
      logseries (positive) pseudo-random integers and <None> is returned.
      Otherwise, a single logarithmic series pseudo-random is returned.
      The algorithm is by A.W. Kemp; see Devroye, 1986, pp 547-548."""
      p = pr
      if not(0.0 < p < 1.0):
         raise ValueError, '<p> must be in (0.0, 1.0)'

      else:
         r = log(1.0 - p)
         i = self._index                  # local alias
         buf = self._ranbuf               # local alias
         buflim = self._ranbuf_size       # local alias
         if size is not None:
            size, Ns = _check_shape(size)
            buffer = Num.zeros(Ns,Num.Float)
            out_len = Ns            
            j = 0

         else:
            out_len = 0
            j = -1

         if p < 0.95:
            pr_one = -p / r
            while j < out_len:
               sum = pr_one
               i = i + 1
               if i == buflim: self._fillbuf(); i = 0
               u = buf[i]
               k = 1
               while u > sum:
                  u = u - sum
                  k = k + 1
                  sum = sum * p * (k - 1) / k

               if out_len:
                  buffer[j] = k
                  j = j + 1

               else:
                  self._index = i
                  return k

         else:                                                 # p is >= 0.95.
            while j < out_len:
               k = 1
               i = i + 1
               if i == buflim: self._fillbuf(); i = 0
               v = buf[i]
               if v < p:
                  i = i + 1
                  if i == buflim: self._fillbuf(); i = 0
                  u = buf[i]
                  q = 1.0 - exp(r * u)
                  q2 = q * q
                  if v <= q2 :
                     k = int( floor(1.0 + log(v) / log(q)) )

                  elif (q2 < v <= q):
                     k = 1

                  else:
                     k = 2

               if out_len:
                  buffer[j] = k
                  j = j + 1

               else:
                  self._index = i
                  return k

         self._index = i  # Restore ranbuf index and return (<buffer> filled).
         return Num.reshape(buffer, size)         

   def _von_Mises(self, b, loc=0.0, size=None):
      """Return von Mises distribution pseudo-random variates on [-pi, +pi].

      von_Mises(b, loc=0.0, size=None)

      <mean> must be in the open interval (-math.pi, +math.pi).  If <buffer>
      is specified, it must be a mutable sequence (list or array).  It is
      filled with von Mises(mean, shape) pseudo-random variates and <None> is
      returned.  Otherwise, a single von Mises RV is returned. The method is
      an algorithm of Best and Fisher, 1979; see Fisher, N. I., "Statistical
      Analysis of Circular Data," Cambridge University Press, 1995, p. 49."""

      shape, mean = b, loc
      z = exp(1j*mean)
      mean = arctan2(z.imag, z.real)
      if not (-3.1415926535897931 < mean < +3.1415926535897931):
         raise ValueError, \
            '<loc> must be in the open interval (-math.pi, math.pi)'

      else:
         a = 1.0 + sqrt( 1.0 + 4.0 * shape*shape )
         b = ( a - sqrt(a + a) ) / (shape + shape)
         r = (1.0 + b * b) / (b + b)
         i = self._index                          # local alias
         buf = self._ranbuf                       # local alias
         buflim = self._ranbuf_size               # local alias
         if size is not None:
            size, Ns = _check_shape(size)
            buffer = Num.zeros(Ns,Num.Float)
            out_len = Ns            
            j = 0

         else:
            out_len = 0
            j = - 1

         while j < out_len:
            while (1):
               i = i + 1
               if i == buflim: self._fillbuf(); i = 0
               z = cos(3.1415926535897931 * buf[i])
               f = (1.0 + r * z) / (r + z)
               c = shape * (r - f)
               i = i + 1
               if i == buflim: self._fillbuf(); i = 0
               u = buf[i]
               if (c * (2.0 - c) - u > 0.0):
                  break                           # quick acceptance

               if log(c / u) + 1.0 - c < 0.0:
                  continue                        # quick rejection

               else:
                  break                           # acceptance

            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            if out_len:
               if buf[i] > 0.5:
                  buffer[j] = fmod(acos(f) + mean, 6.2831853071795862)

               else:
                  buffer[j] = -fmod(acos(f) + mean, 6.2831853071795862)

               j = j + 1

            else:
               self._index = i # Restore updated value of _ranbuf index.
               if buf[i] > 0.5: return  fmod(acos(f)+mean, 6.2831853071795862)
               else:            return -fmod(acos(f)+mean, 6.2831853071795862)

         self._index = i # Restore _ranbuf index and return (buffer[] filled).
         return Num.reshape(buffer, size)
               
   def _Wald(self, mu, loc=0.0, scale=1.0, size=None):
      """Return Inverse gaussian pseudo-random variates.

      invnorm(mu, loc=0.0, scale=1.0, size=None)

      Both <mean> and <scale> must be positive.  If <buffer> is specified, it
      must be a mutable sequence (list or array).  It is filled with inverse
      gaussian (wald if mu=1.0) pseudo-random variates, and <None> is returned.
      Otherwise, a single Wald pseudo-random variate is returned."""
      mean = mu
      oscale = scale
      scale = 1.0
      if (mean <= 0.0) or (scale <= 0.0):
         raise ValueError, 'both <mean> and <scale> must be positive'

      else:
         i = self._index                          # local alias
         buf = self._ranbuf                       # local alias
         buflim = self._ranbuf_size               # local alias
         if size is not None:
            size, Ns = _check_shape(size)
            buffer = Num.zeros(Ns,Num.Float)
            out_len = Ns            
            j = 0

         else:
            out_len = 0
            j = -1

         normal = self._normal                    # local alias
         r = 0.5 * mean / scale
         con = 4.0 * scale
         meansq = mean * mean
         while j < out_len:
            n = normal()
            muy = mean * n * n
            x1 = mean + r * ( muy -  sqrt(muy * (con + muy)) )
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            if out_len:
               if buf[i] <= mean / (mean + x1):
                  buffer[j] = x1

               else:
                  buffer[j] =  meansq / x1

               j = j + 1

            else:
               self._index = i       # Restore updated value of _ranbuf index.
               if buf[i] <= mean / (mean + x1):
                  return x1

               else:
                  return meansq / x1

      self._index = i    # Restore _ranbuf index and return (buffer[] filled).
      return Num.reshape(buffer*oscale+loc, size)      
      
   def _Zipf(self, a=4.0, size=None):
      """Return positive pseudo-random integers from the Zipf distribution

      Zipf(a=4.0, size=None)

      Z has the Zipf ditribution with parameter a if
         Pr{Z=i} = 1.0 / (zeta(a) * i ** a) for i = 1, 2, ... and a > 1.0.
      Here zeta(a) is the Riemann zeta function, the sum from 1 to infinity of
      1.0 / i **a.  If <buffer> is specified, it must be a mutable sequence
      (list or array).  It is filled with Zipf pseudo-random integers and
      <None> is returned.  Otherwise, a single (positive) Zipf pseudo-random
      integer is returned; see Devroye, 1986, pp 550-551 for the algorithm."""
      if a <= 1.0:
         raise ValueError, '<a> must be larger than 1.0'

      else:
         i = self._index                  # local alias
         buf = self._ranbuf               # local alias
         buflim = self._ranbuf_size       # local alias
         am1 = a - 1.0
         b = 2.0 ** am1
         if size is not None:
            size, Ns = _check_shape(size)
            buffer = Num.zeros(Ns,Num.Float)
            out_len = Ns
            j = 0

         else:
            out_len = 0
            j = -1

         while j < out_len:
            while 1:
               i = i + 1
               if i == buflim: self._fillbuf(); i = 0
               x = floor( buf[i] ** (-1 / am1) )
               t = (1.0 + 1.0 / x) ** am1
               i = i + 1
               if i == buflim: self._fillbuf(); i = 0
               if buf[i] * x * (t - 1.0) / (b - 1.0)  <= t / b:
                  break

            if out_len:
               buffer[j] = int(x)
               j = j + 1

            else:
               self._index = i
               return int(x)

         self._index = i  # Restore ranbuf index and return (buffer[] filled).
         return Num.reshape(buffer, size)         

#               End of Non-uniform(0,1) Generators

#               Geometric and Subsampling Routines

   def _in_simplex(self, mseq=5*[0.0], bound=1.0):
      """Return a pseudorandom point in a simplex.

      in_simplex(mseq=2*[0.0], bound=1.0)

      The simplex of dimension d and bound b > 0 is defined by all points
      z = (z1, z2, ..., zd) with z1 + z2 + ... + zd <= b and all zi >= 0.
      <bound> must be positive.
      A two-dimensional simplex with bound b is the region enclosed by the
      horizontal (x) and vertical (y) axes and the line y = b - x.  A
      one dimensional simplex with bound b is the single point b.
      <mseq> must be a mutable sequence (list or array); if it is an array, it
      must be typed double or single float.  A sequence of the same type and
      length as <mseq> containing the coordinates of a pseudo-random point in
      the simplex is returned.
      See Fishman, George, "Monte Carlo," Springer-Verlag, 1996, pp. 232-233
      and 226-229.  Algorithms EUNIFORM and OSIMP were used."""
      d = len(mseq)
      point = mseq[:]
      if bound <= 0.0:
         raise ValueError, '<bound> must be positive.'

      elif d == 1:
         point[0] = bound

      else:
         i = self._index                          # local alias
         buf = self._ranbuf                       # local alias
         buflim = self._ranbuf_size               # local alias
         range_d = range(d)
         i = i + 1
         if i == buflim: self._fillbuf(); i = 0
         point[0] = -log(buf[i])
         for j in range_d[1:]: # Accumulate running sum of exponential(1) RVs.
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            point[j] = point[j-1] - log(buf[i])

         i = i + 1
         if i == buflim: self._fillbuf(); i = 0
         total = point[d-1] - log(buf[i])
         for j in range_d:
            point[j] = point[j] / total

         point[0] = point[0] * bound
         j = d - 1
         while j > 0:
            point[j] = bound * (point[j] - point[j-1])
            j = j - 1

      self._index = i     # Restore updated value of _ranbuf index and return.
      return point


   def _in_sphere(self, center=5*[0.0], radius=1.0):
      """Return a pseudo-random point in a sphere centered at <center>.

      in_sphere(center=5*[0.0], radius=1.0)

      A one-dimensional sphere centered at 0 is the two points <+radius> and
      <-radius>.  A two-dimensional sphere centered at 0 is the circle with
      radius <radius>.  <center> must be a sequence containing double
      coordinates for the sphere's center. <radius> must be positive.
      pseudo-random point in the "sphere" with center 0 and radius <radius>.
      See Fishman, George, "Monte Carlo," Springer-Verlag, 1996, pp 234-235.
      Algorithm OSPHERE was used."""
      d = len(center)
      if radius <= 0.0:
         raise ValueError, '<radius> must be positive.'

      else:
         return self._on_sphere( center, radius * self._beta(float(d), 1.0) )

   def _on_simplex(self, mseq=5*[0.0], bound=1.0):
      """Return a pseudo_random point on the boundary of a simplex.

      on_simplex(mseq=5*[0.0], bound=1.0)

      The simplex of dimension d and bound b > 0 is defined by all vectors
      z = (z1, z2, ..., zd) with z1 + z2 + ... + zd <= b and all zi >= 0.
      <mseq> must be a mutable sequence (list or array) and <bound> must be
      positive.  A boundary point on the simplex has the sum of its
      coefficients equal to b.
      The coordinates of a pseudo-random point on the boundary of the
      simplex of dimension len(seq) and bound <bound> are returned as a
      sequence of the same type and length as <mseq>.  The input values in
      <mseq> are ignored, and undisturbed.
      See Fishman, George, "Monte Carlo," Springer-Verlag, 1996,
      pp 232-233 and 226-229.  Algorithms EUNIFORM and OSIMP were used."""
      d = len(mseq)
      point = mseq[:]
      if bound <= 0.0:
         raise ValueError, '<bound> must be positive.'

      elif d == 1:                         # The point is just <bound>.
         point[0] = bound
      else:
         i = self._index                           # local alias
         buf = self._ranbuf                        # local alias
         buflim = self._ranbuf_size                # local alias
         range_d = range(d)                        # [0,1,...,d-1]
         range_dm1 = range_d[:(d - 1)]             # [0,1,...,d-2]
         range_1_dm1 = range_dm1[1:]               # [1,2,...,d-2]
         i = i + 1
         if i == buflim: self._fillbuf(); i = 0
         point[0] = -log(buf[i])
         for j in range_1_dm1:             # range_1_dm1 is empty for d = 2.
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            point[j] = point[j - 1] - log(buf[i])

         i = i + 1
         if i == buflim: self._fillbuf(); i = 0
         total = point[d - 2] - log(buf[i])
         for j in range_dm1:
            point[j] = point[j] / total

         last = point[d - 2]
         point[0] = bound * point[0]
         j = d - 2
         while j > 0:                      # not executed for d = 2.
            point[j] = bound * (point[j] - point[j - 1])
            j = j - 1

         point[d - 1] = bound - last

      self._index = i     # Restore updated value of _ranbuf index and return.
      return point

   def _on_sphere(self, center=5*[0.0], radius=1.0):
      """Return a pseudo-random point on the sphere centered at point[].

      on_sphere(center=5*[0.0], radius=1.0)

      The sphere of dimension d centered at 0 is all points (x1,x2,...,xd)
      with x1**2 + x2**2 + ... + xd**2 <= <radius>**2.  The surface of this
      sphere is any point with the sum of its components squared equal to
      the radius squared.  A 2-dimensional sphere is a circle, and a
      1-dimensional sphere is the line from <-radius> to <+radius>.  <center>
      must be a sequence and <radius> must be positive.  A sequence of the
      same type as <center[]> containing the coordinates of a pseudo-random
      point on the "sphere" centered at <center[]> is returned.
      See Fishman, George, "Monte Carlo," Springer-Verlag, 1996, pp 234-235.
      Algorithm OSPHERE was used.  For dimensions 2, 3, and 4, rejection
      algorithms were used. See Marsaglia, G., "Choosing a point from the
      surface of a sphere," Annals of mathematical Statistics, vol. 43,
      pp. 645-646, 1972."""
      d = len(center)
      buflim = self._ranbuf_size        # local alias
      buf = self._ranbuf                # local alias
      i = self._index                   # local alias
      point = center[:]
      if  radius <= 0.0:
         raise ValueError, '<radius> must be positive.'

      elif d == 0:                      # Just return center[:].
         pass

      elif d == 1:
         i = i + 1
         if i == buflim: self._fillbuf(); i = 0
         if buf[i] > 0.5:
            point[0] = radius + center[0]

         else:
            point[0] = -radius + center[0]

      elif d == 2:                      # rejection from circumscribed square.
         ss = 2.0
         while ss > 1.0:
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            x = buf[i]
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            y = buf[i]
            x = x + x - 1.0                # (x, y) is in the square circum-
            y = y + y - 1.0                # scribed on the unit circle
            ss = x * x + y * y             # centered at (0, 0).

         ss = radius / sqrt(ss)            # (x, y) now restricted to unit
         point[0] = x * ss + center[0]     # circle.
         point[1] = y * ss + center[1]
         self._index = i                   # Restore ranbuf index and return.

      elif d == 3:                         # algorithm by Marsaglia, 1972
         ss = 2.0
         while ss > 1.0:
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            u = buf[i]
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            v = buf[i]
            u = u + u - 1.0
            v = v + v - 1.0
            ss = u * u + v * v

         con = sqrt(1.0 - ss) * radius
         point[0] = 2.0 * u * con + center[0]
         point[1] = 2.0 * v * con + center[1]
         point[2] = (1.0 - ss - ss) * radius + center[2]
         self._index = i                   # Restore ranbuf index and return.

      elif d == 4:
         ssuv = 2.0                        # algorithm by Marsaglia, 1972
         while ssuv > 1.0:
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            u = buf[i]
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            v = buf[i]
            u = u + u - 1.0
            v = v + v - 1.0
            ssuv = u * u + v * v

         ssrs = 2.0
         while ssrs > 1.0:
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            r = buf[i]
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            s = buf[i]
            r = r + r - 1.0
            s = s + s - 1.0
            ssrs = r * r + s * s

         con = sqrt((1.0 - ssuv) / ssrs) * radius
         point[0] = u * radius + center[0]
         point[1] = v * radius + center[1]
         point[2] = r * con + center[2]
         point[3] = s * con + center[3]
         self._index = i                   # Restore ranbuf index and return.

      else:                                # Use radially symmetric normals
         range_d = range(d)                # (Fishmans algorithm OSPHERE).
         sum = 0.0
         normal = self._normal
         for j in range_d:
            z = normal()
            sum = sum + z * z
            point[j] = z

         constant = radius / sqrt(sum)
         for j in range_d:
           point[j] = constant * point[j] + center[j]

      return point

   def _sample(self, inlist=[0,1,2], sample_size=2):
      """Return simple random sample of <sample_size> items from <inlist>.

      sample(inlist=[0,1,2], sample_size=2)

      <inlist> must be a mutable sequence (list or array).  The
      <sample_size> <= len(inlist) elements of the returned mutable sequence
      (which has the same type as <inlist>) are a simple random sample from
      all the elements in <inlist>.
      Sampling is with replacement; duplicates may occur in the returned
      sequence.  The length of the returned list or array is <sample_size>,
      which may be any positive integer."""
      pop_size = len(inlist)
      n = int(sample_size)
      if pop_size == 0:
         outlist = inlist[:]

      elif n <= 0:
         outlist = 0 * inlist[:1]

      else:
         i = self._index                          # local alias
         buf = self._ranbuf                       # local alias
         buflim = self._ranbuf_size               # local alias
         outlist = n*inlist[:1]                   # <sample_size> >= 1
         for j in xrange(n):
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            outlist[j] = inlist[int( floor(buf[i] * pop_size) )]

      self._index = i                     # Restore _ranbuf index arnd return.
      return outlist

   def _smart_sample(self, inlist=[0,1,2], sample_size=2):
      """Return a random sample without replacement of elements in <inlist>.

      smart_sample(inlist=[0,1,2], sample_size=2)

      <inlist> must be a mutable sequence (list or array).  The
      <sample_size> <= len(inlist) elements of the returned mutable sequence
      (which has the same type as <inlist>) are a sample without replacement
      of all the elements in <inlist>.  Since sampling is without replacement,
      there is no possibility of duplicates in the returned list or array
      unless there are duplicates in <inlist>.  The length of the
      returned mutable sequence is the smaller of <sample_size> and
      len(inlist), and its type is the type of <inlist>."""
      pop_size = len(inlist)
      n = int( min(sample_size, pop_size) )  # can't have sample > population.
      if pop_size <= 1:
         outlist = inlist[:]

      elif n < 1:
         outlist = 0*inlist[:1]

      else:
         outlist = inlist[:]            # <outlist> has same type as <inlist>.
         i = self._index                # local alias
         buf = self._ranbuf             # local alias
         buflim = self._ranbuf_size     # local alias
         for j in xrange(n):
            temp = outlist[j]           # Save item in ith position.
            i = i + 1
            if i == buflim: self._fillbuf(); i = 0
            k = j + int( floor(buf[i] * (pop_size - j)) )
            outlist[j] = outlist[k]     # Move sampled item to ith position.
            outlist[k] = temp           # copy to sampled item's position.

         if n < pop_size:
            del outlist[n:pop_size]     # Remove any unsampled elements.

      self._index = i                   # Restore _ranbuf index and return.
      return outlist

_inst = _pranv()   # Initialize uniform(0,1) generator to default; clock seed.

initial_seed      =  _inst._initial_seed
initialize        =  _inst._initialize
random_algorithm  =  _inst._random_algorithm
random_count      =  _inst._random_count
save_state        =  _inst._save_state

choice            =  _inst._choice

#     Geometrical Point Generators, Permutations and Subsampling Routines
in_simplex        =  _inst._in_simplex
in_sphere         =  _inst._in_sphere
on_simplex        =  _inst._on_simplex
on_sphere         =  _inst._on_sphere
sample            =  _inst._sample
smart_sample      =  _inst._smart_sample


##def mean_var_test(x, type, mean, var, skew=[]):
##    n = len(x) * 1.0
##    x_mean = Num.sum(x)/n
##    x_minus_mean = x - x_mean
##    x_var = Num.sum(x_minus_mean*x_minus_mean)/(n-1.0)
##    print "\nAverage of ", len(x), type
##    print "(should be about ", mean, "):", x_mean
##    print "Variance of those random numbers (should be about ", var, "):", x_var
##    if skew != []:
##       x_skew = (Num.sum(x_minus_mean*x_minus_mean*x_minus_mean)/9998.)/x_var**(3./2.)
##       print "Skewness of those random numbers (should be about ", skew, "):", x_skew

##def test():
##    x, y = get_seed()
##    print "Initial seed", x, y
##    seed(x, y)
##    x1, y1 = get_seed()
##    if x1 != x or y1 != y:
##        raise SystemExit, "Failed seed test."
##    print "First random number is", random()
##    print "Average of 10000 random numbers is", Num.sum(random(10000))/10000.
##    x = random([10,1000])
##    if len(x.shape) != 2 or x.shape[0] != 10 or x.shape[1] != 1000:
##        raise SystemExit, "random returned wrong shape"
##    x.shape = (10000,)
##    print "Average of 100 by 100 random numbers is", Num.sum(x)/10000.
##    y = uniform(0.5,0.6, (1000,10))
##    if len(y.shape) !=2 or y.shape[0] != 1000 or y.shape[1] != 10:
##        raise SystemExit, "uniform returned wrong shape"
##    y.shape = (10000,)
##    if Num.minimum.reduce(y) <= 0.5 or Num.maximum.reduce(y) >= 0.6:
##        raise SystemExit, "uniform returned out of desired range"
##    print "randint(1, 10, size=[50])"
##    print randint(1, 10, size=[50])
##    print "permutation(10)", permutation(10)
##    print "randint(3,9)", randint(3,9)
##    print "random_integers(10, size=[20])"
##    print random_integers(10, size=[20])
##    s = 3.0
##    x = norm(2.0, s, [10, 1000])
##    if len(x.shape) != 2 or x.shape[0] != 10 or x.shape[1] != 1000:
##        raise SystemExit, "standard_normal returned wrong shape"
##    x.shape = (10000,)
##    mean_var_test(x, "normally distributed numbers with mean 2 and variance %f"%(s**2,), 2, s**2, 0)
##    x = exponential(3, 10000)
##    mean_var_test(x, "random numbers exponentially distributed with mean %f"%(s,), s, s**2, 2)
##    x = multivariate_normal(Num.array([10,20]), Num.array(([1,2],[2,4])))
##    print "\nA multivariate normal", x
##    if x.shape != (2,): raise SystemExit, "multivariate_normal returned wrong shape"
##    x = multivariate_normal(Num.array([10,20]), Num.array([[1,2],[2,4]]), [4,3])
##    print "A 4x3x2 array containing multivariate normals"
##    print x
##    if x.shape != (4,3,2): raise SystemExit, "multivariate_normal returned wrong shape"
##    x = multivariate_normal(Num.array([-100,0,100]), Num.array([[3,2,1],[2,2,1],[1,1,1]]), 10000)
##    x_mean = Num.sum(x)/10000.
##    print "Average of 10000 multivariate normals with mean [-100,0,100]"
##    print x_mean
##    x_minus_mean = x - x_mean
##    print "Estimated covariance of 10000 multivariate normals with covariance [[3,2,1],[2,2,1],[1,1,1]]"
##    print Num.matrixmultiply(Num.transpose(x_minus_mean),x_minus_mean)/9999.
##    x = beta(5.0, 10.0, 10000)
##    mean_var_test(x, "beta(5.,10.) random numbers", 0.333, 0.014)
##    x = gamma(.01, 2., 10000)
##    mean_var_test(x, "gamma(.01,2.) random numbers", 2*100, 2*100*100)
##    x = chi_square(11., 10000)
##    mean_var_test(x, "chi squared random numbers with 11 degrees of freedom", 11, 22, 2*Num.sqrt(2./11.))
##    x = F(5., 10., 10000)
##    mean_var_test(x, "F random numbers with 5 and 10 degrees of freedom", 1.25, 1.35)
##    x = poisson(50., 10000)
##    mean_var_test(x, "poisson random numbers with mean 50", 50, 50, 0.14)
##    print "\nEach element is the result of 16 binomial trials with probability 0.5:"
##    print binomial(16, 0.5, 16)
##    print "\nEach element is the result of 16 negative binomial trials with probability 0.5:"
##    print negative_binomial(16, 0.5, [16,])
##    print "\nEach row is the result of 16 multinomial trials with probabilities [0.1, 0.5, 0.1 0.3]:"
##    x = multinomial(16, [0.1, 0.5, 0.1], 8)
##    print x
##    print "Mean = ", Num.sum(x)/8.

##if __name__ == '__main__': 
##    test()



""" Test functions for basic module

"""

import unittest
import scipy_base.limits as limits
from scipy_test.testing import assert_array_equal, assert_equal
from scipy_test.testing import assert_almost_equal, assert_array_almost_equal
from scipy_base import sqrt, product, add,  ravel, mgrid
from scipy.common import rand,randn,comb,factorial


##################################################

### Test for rand

class test_rand(unittest.TestCase):
    def __init__(self,*args,**kwds):
        unittest.TestCase.__init__(self,*args,**kwds)
        self.z = rand(100,10,20)
        
    def check_shape(self):
        assert_equal(self.z.shape,(100,10,20))

    def check_mean(self):
        mn = add.reduce(ravel(self.z)) / 200 / 100.0
        assert_almost_equal(mn,0.5,1)

    def check_std(self):
        std = add.reduce((ravel(self.z)-0.5)**2) / 200.0 / 100.0
        assert_almost_equal(std,1.0/12.0,2)

class test_randn(unittest.TestCase):
    def __init__(self,*args,**kwds):
        unittest.TestCase.__init__(self,*args,**kwds)
        self.z = randn(100,10,20)
        
    def check_shape(self):
        assert_equal(self.z.shape,(100,10,20))

    def check_mean(self):
        mn = add.reduce(ravel(self.z)) / 200 / 100.0
        assert_almost_equal(mn,0.0,1)

    def check_std(self):
        std = sqrt(add.reduce(ravel(self.z)**2) / 200.0 / 100.0)
        assert_almost_equal(std,1.0,1)

class test_factorial(unittest.TestCase):
    def check_basic(self):
        for k in range(0,13):
            assert_equal(factorial(k),product(mgrid[1:k+1]))
        assert_equal(factorial(-10),0)

    def check_exact(self):
        resdict = {4:24L,10:3628800L,15:1307674368000L,
                   19:121645100408832000L, -10:0L,
                   100:93326215443944152681699238856266700490715968264381621468592963895217599993229915608941463976156518286253697920827223758251185210916864000000000000000000000000L}
        for key in resdict.keys():
            assert_equal(factorial(key,exact=1),resdict[key])

class test_comb(unittest.TestCase):
    def check_basic(self):
        for N in range(0,11):
            for k in range(0,N+1):
                ans = product(mgrid[N-k+1:N+1]) / product(mgrid[1:k+1])
                assert_almost_equal(comb(N,k),ans,9)
        assert_equal(comb(-10,1),0)
        assert_equal(comb(10,-1),0)
        assert_equal(comb(-10,-3),0)
        assert_equal(comb(10,11),0)

    def check_exact(self):
        resdict = {(10,2):45L, (10,5):252L,
                   (1000,20):339482811302457603895512614793686020778700L,
                   (1000,975):47641862536236518640933948075167736642053976275040L,
                   (-10,1):0L, (10,-1):0L, (-10,-3):0L,(10,11):0L}
        for key in resdict.keys():
            assert_equal(comb(key[0],key[1],exact=1),resdict[key])
        
#-----------------------------------------------------------------------------

def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append( unittest.makeSuite(test_rand,'check_') )
        suites.append( unittest.makeSuite(test_randn,'check_') )
        suites.append( unittest.makeSuite(test_factorial,'check_') )
        suites.append( unittest.makeSuite(test_comb,'check_') )
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite(level)
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner


if __name__ == "__main__":
    test()



from scipy_base import *
from scipy_base.fastumath import *

def myasarray(a):
    if type(a) in [type(1.0),type(1L),type(1),type(1j)]:
        a = asarray([a])
    if len(a) == 0:
        a = asarray([a])
    return a

def check_func(thefunc, x0, args, numinputs, output_shape=None):
    args = (x0[:numinputs],) + args
    res = myasarray(apply(thefunc,args))
    if (output_shape != None) and (shape(res) != output_shape):
        if (output_shape[0] != 1):
            if len(output_shape) > 1:
                if output_shape[1] == 1:
                    return shape(res)
            raise TypeError, "There is a mismatch between the input and output shape of %s." % thefunc.func_name
    return shape(res)






#
# xxx - Example of a Scipy module, see DEVELOPERS.txt 
#

# Get documentation string:
from info_xxx import __doc__

# Import symbols from sub-module:
from foo import *

# Import sub-package:
import yyy

# Import extension module
import spam



#!/usr/bin/env python

import os

def configuration(parent_package='', parent_path=None):

    # The following three lines constitute minimal contents
    # of configuration(..) that is suitable for pure Python
    # packages.
    package = 'xxx'
    from scipy_distutils.misc_util import default_config_dict, dot_join, get_path
    config = default_config_dict(package,parent_package)

    # local_path should be joined with local source files
    # when specifying sources below
    local_path = get_path(__name__)

    # xxx contains pure Python sub-package yyy
    config['packages'].append(dot_join(parent_package,package,'yyy'))
    # And don't forget adding sub-packages of yyy:
    config['packages'].append(dot_join(parent_package,package,'yyy.tests'))

    # xxx has extension module ..
    from scipy_distutils.core import Extension

    
    # xxx has f2py generated extension module ..
    # xxx has swig generated extension module ..

    # xxx generates source code
    from scipy_distutils.misc_util import SourceGenerator
    def generate_spam_pyf(target, sources):
        fin = open(sources[0])
        body = fin.read()
        fin.close()
        fout = open(target,'w')
        fout.write('python module spam\n%s\nend python module spam' % body)
        fout.close()
    pyf_file = SourceGenerator(generate_spam_pyf,
                               target='spam.pyf',
                               sources=[os.path.join(local_path,'spam_src.pyf')])
    ext = Extension(name=dot_join(parent_package,package,'spam'),
                    sources=[pyf_file],
                    depends = pyf_file.sources)
    config['ext_modules'].append(ext)

    return config

if __name__ == '__main__':
    from scipy_distutils.core import setup
    setup(**configuration())



#
# ga - Genetic Algorithms
#

#from pre___init__ import __doc__

import tree
import algorithm
import ga_util
import ga_list
import gene
import genome
import language
import population
import scaling
import selection
import tree_opt

# need to look at ga_gnm, tree_opt to see if they really fit.



import gui_thread
from wxPython.wx import *

def modal_sample():
    import gui_thread
    prxyDirDialog = gui_thread.register(wxDirDialog)
    q=prxyDirDialog(None)
    q.ShowModal()
    
class TrackingEvtHandler:
    """ Used occasionly for debugging which handlers are 
        connected
    """
    def Connect( self, *arguments, **namedarguments ):
        print 'arguments', arguments, namedarguments
        if not hasattr( self, "__ids" ):
            self.__ids = []
        self.__ids.append( arguments[0] )
        return apply( wxEvtHandler.Connect, (self,)+arguments,namedarguments )
    def ReleaseEventHandlers( self ):
        while self.__ids:
            self.Disconnect( self.__ids[-1] )
            del self.__ids[-1]

#class SimpleFrame(TrackingEvtHandler, wxFrame):
class SimpleFrame( wxFrame):
    def __init__(self, parent=None, ID=-1, title='no title', 
                 pos=wxDefaultPosition, size=wxSize(350,200)):
        wxFrame.__init__(self, parent, ID, title, pos, size)
        panel = wxPanel(self, -1)

        button = wxButton(panel, 1003, "Close Me")
        button.SetPosition(wxPoint(15, 15))
        self.button = button       
        EVT_BUTTON(self, 1003, self.OnCloseMe)
        EVT_CLOSE(self, self.OnCloseWindow)
        # Ah! The next line is ignored.  It would seem you must Disconnect
        # events before adding new ones.
        EVT_CLOSE(self, self.OnCloseWindow2)
        self.name = 'bubba'
        
    def Blue(self):
        self.button.SetBackgroundColour(wxBLUE)

    def Red(self):
        self.button.SetBackgroundColour(wxRED)

    def OnCloseMe(self, event):
        print 'hit'
        self.Close(true)

    def OnCloseWindow(self, event):
        print 'first called'
        self.Destroy()

    def OnCloseWindow2(self, event):
        print '2nd called'
        self.Destroy()

def non_modal_sample():
    prxySimpleFrame = gui_thread.register(SimpleFrame)
    win = prxySimpleFrame(None, -1, "This is a wxFrame", wxDefaultPosition, 
                      wxSize(350, 200))
    win.Show(true)
    return win